<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fdl="http://www.ibm.com/workflow/2003" xmlns:xalan="http://xml.apache.org/xalan" xmlns:java="http://xml.apache.org/xalan/java" extension-element-prefixes="xalan java" exclude-result-prefixes="fdl" xmlns="http://www.w3.org/2001/XMLSchema" version="1.0">
  <!--
    *************************************************************************

      Workfile: BPCSRVR/ws/code/workflow.fdl2bpel/src/com/ibm/bpe/fdl2bpel/converter/v62/xfdl2bpel_analyze.xsl, BPC.workflow, WBI70.BPCSRVR, of0950.12
      Last update: 09/01/14 03:54:25
      SCCS path, id: /family/botp/vc/13/4/8/5/s.55 1.10

    *************************************************************************
    BEGIN COPYRIGHT
    *************************************************************************

      IBM Confidential
      OCO Source Materials
      5655-FLW (C) Copyright IBM Corporation 2003, 2008.
      The source code for this program is not published or otherwise
      divested of its trade secrets, irrespective of what has been
      deposited with the U.S. Copyright Office.

    *************************************************************************
    END COPYRIGHT
  -->
  <!--
    *****************************************************************************************************************************
          Task:     Analyze the XFDL file that was generated from the FDL input file
    *****************************************************************************************************************************
          Author: Jens Grotrian (jgr@de.ibm.com)
    *****************************************************************************************************************************
   -->
  <xsl:strip-space elements="*"/>
  <xsl:output method="html" indent="yes" xalan:indent-amount="5"/>
  <!--************************************ -->
  <!--*        Configurable parameters   * -->
  <!--************************************ -->
  <!-- "fdlFileName": Name of FDL file -->
  <xsl:param name="fdlFileName" select="'unknown_fdlFileName'"/>
  <!-- "createUPESInvocations" (cp. LI754: Do not prepare process for reuse of UPES ) -->
  <xsl:param name="createUPESInvocations" select="'TRUE'"/>
  <!-- "LI1274_Support_inheritance_from_network_settings" -->
  <xsl:param name="LI1274_Support_inheritance_from_network_settings" select="'TRUE'"/>
  <!--************************** -->
  <!--*        Parameters         * -->
  <!--************************** -->
  <!-- "end of line" character string -->
  <xsl:param name="eol" select="java:java.lang.System.getProperty('line.separator')"/>
  <!-- "lower-case-characters": "aäbcdefghijklmnoöpqrstuüvwxyz" -->
  <xsl:param name="lower-case-characters" select="'aäbcdefghijklmnoöpqrstuüvwxyz'"/>
  <!-- "Upper-case-characters": "AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ" -->
  <xsl:param name="upper-case-characters" select="'AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ'"/>
  <!-- "Column-1-width": Width of first table column -->
  <xsl:param name="Column-1-width" select="100"/>
  <!-- "Column-2-width": Width of second table column -->
  <xsl:param name="Column-2-width" select="1600"/>
  <!--************************** -->
  <!--*        Keys                   * -->
  <!--************************** -->
  <!--
    Key "server-by-server-and-system-name" represents a "mapping" between the concatenated names of server and related system (as "key objects") and server definitions (as "value objects")
  -->
  <xsl:key name="server-by-server-and-system-name" match="fdl:definitions/fdl:server
                                                                    | fdl:definitions/fdl:create/fdl:server
                                                                    | fdl:definitions/fdl:update/fdl:server
                                                                    | fdl:definitions/fdl:replace/fdl:server" use="concat(@name, fdl:relatedSystem/@name)"/>
  <!--************* -->
  <!-- Templates   -->
  <!--************* -->
  <!--
    *****************
    Initial template
    *****************
  -->
  <xsl:template match="/">
    <!-- Execute examination rules applicable to "fdl:definitions" -->
    <xsl:apply-templates select="fdl:definitions" mode="examine"/>
  </xsl:template>
  <!--
    *************************************************************
    Examine "fdl:definitions"
    *************************************************************
  -->
  <xsl:template match="fdl:definitions" mode="examine">
    <HTML>
      <BODY>
        <H1>FDL Examination Report</H1>
        <TABLE border="1">
          <TBODY>
            <TR bgcolor="#c0c0c0">
              <TD width="{$Column-1-width}">
                <B>FDL</B>
              </TD>
              <TD width="{$Column-2-width}">
                <B>
                  <xsl:value-of select="$fdlFileName"/>
                </B>
              </TD>
            </TR>
            <!-- Execute examination rules applicable to "fdl:process" -->
            <xsl:apply-templates select="fdl:process | fdl:create/fdl:process | fdl:update/fdl:process | fdl:replace/fdl:process" mode="examine"/>
          </TBODY>
        </TABLE>
      </BODY>
    </HTML>
  </xsl:template>
  <!--
    *************************************************************
    Examine "fdl:process"
    *************************************************************
  -->
  <xsl:template match="fdl:process" mode="examine">
    <!-- Apply examination rule 1 -->
    <xsl:apply-templates select="//fdl:programActivity | //fdl:processActivity" mode="Rule-1">
      <xsl:with-param name="process-name" select="@displayName"/>
    </xsl:apply-templates>
    <!-- Apply examination rule 2 -->
    <xsl:apply-templates select="//fdl:programActivity" mode="Rule-2">
      <xsl:with-param name="process-name" select="@displayName"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *************************************************************
    FDL EXAMINATION RULE 1:

      Look for missing "DONE_BY" or "DONE_BY ALL"
    *************************************************************
  -->
  <xsl:template match="fdl:programActivity | fdl:processActivity" mode="Rule-1">
    <xsl:param name="process-name"/>
    <xsl:choose>
      <xsl:when test="fdl:staff">
        <xsl:if test="fdl:staff/fdl:all">
          <xsl:variable name="text">
            <xsl:value-of select="concat('In process &quot;', $process-name, '&quot; the activity &quot;', @displayName, '&quot; there is the following staff assignment specified: &quot;All people&quot; (FDL: &quot;DONE_BY ALL&quot;). ')"/>
            <xsl:value-of select="'Be aware that &quot;DONE BY ALL&quot; may have severe impact on system performance!'"/>"
          </xsl:variable>
          <xsl:call-template name="display-hint">
            <xsl:with-param name="title" select="'Hint 1'"/>
            <xsl:with-param name="text" select="$text"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="text">
          <xsl:value-of select="concat('In process &quot;', $process-name, '&quot; the activity &quot;', @displayName, '&quot; there is no staff assignment specified. ')"/>
          <xsl:value-of select="'By default the process server will assume &quot;DONE BY ALL&quot; as staff assignment rule, which means that staff resolution will include all users. '"/>
          <xsl:value-of select="'Be aware that &quot;DONE BY ALL&quot; may have severe impact on system performance!'"/>
        </xsl:variable>
        <xsl:call-template name="display-hint">
          <xsl:with-param name="title" select="'Hint 1'"/>
          <xsl:with-param name="text" select="$text"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
    *************************************************************
    FDL EXAMINATION RULE 2:

      Look for improperly defined "empty activities"
    *************************************************************
  -->
  <xsl:template match="fdl:programActivity" mode="Rule-2">
    <xsl:param name="process-name"/>
    <xsl:variable name="uppercase-process-name" select="translate($process-name, $lower-case-characters, $upper-case-characters)"/>
    <xsl:variable name="uppercase-activity-name" select="translate(@displayName, $lower-case-characters, $upper-case-characters)"/>
    <xsl:variable name="uppercase-UPES-name" select="translate(fdl:program/@programExecutionUnit, $lower-case-characters, $upper-case-characters)"/>
    <xsl:variable name="uppercase-program-name" select="translate(fdl:program/@displayName, $lower-case-characters, $upper-case-characters)"/>
    <xsl:variable name="program-activity-type">
      <xsl:call-template name="program-activity-type">
        <xsl:with-param name="activity" select="current()"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Program activity is not of type "empty activity" -->
    <xsl:if test="not($program-activity-type = 'empty activity')">
      <xsl:if test="contains($uppercase-activity-name, 'NOOP')
                    or contains($uppercase-activity-name, 'NO_OP')
                    or contains($uppercase-activity-name, 'NO-OP')
                    or contains($uppercase-activity-name, 'EMPTY')
                    or contains($uppercase-activity-name, 'DUMMY')
                    or contains($uppercase-UPES-name, 'NOOP')
                    or contains($uppercase-UPES-name, 'NO_OP')
                    or contains($uppercase-UPES-name, 'NO-OP')
                    or contains($uppercase-UPES-name, 'EMPTY')
                    or contains($uppercase-UPES-name, 'DUMMY')
                    or contains($uppercase-program-name, 'FMCINTERNALNOOP')
                    or contains($uppercase-program-name, 'NOOP')
                    or contains($uppercase-program-name, 'NO_OP')
                    or contains($uppercase-program-name, 'NO-OP')
                    or contains($uppercase-program-name, 'EMPTY')
                    or contains($uppercase-program-name, 'DUMMY')">
        <xsl:variable name="text">
          <xsl:value-of select="concat('Maybe that in process &quot;', $process-name, '&quot; the activity &quot;', @displayName, '&quot; is an improperly defined &quot;empty activity&quot;! ')"/>
          <xsl:value-of select="'Consult section &quot;Inserting an empty activity in a process model&quot; of Chapter 5 &quot;Advanced modeling techniques&quot; of the book &quot;Getting Started with Buildtime&quot; and modify your process model accordingly, if necessary.'"/>
        </xsl:variable>
        <xsl:call-template name="display-hint">
          <xsl:with-param name="title" select="'Hint 2'"/>
          <xsl:with-param name="text" select="$text"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <!--
    *************************************************************
    Template "display-hint"

      Renders a table row with the text of a user hint.
    *************************************************************
  -->
  <xsl:template name="display-hint">
    <xsl:param name="title"/>
    <xsl:param name="text"/>
    <TR>
      <TD width="{$Column-1-width}">
        <xsl:value-of select="$title"/>
      </TD>
      <TD width="{$Column-2-width}">
        <xsl:value-of select="$text"/>
      </TD>
    </TR>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
    Template "program-activity-type":

    Determines the type of an FDL program activity.
    Selects from types "empty activity", "service invocation activity" (UPES activity),  "staff activity",
    "assumed staff activity"
    (NOTE THAT THIS TEMPLATE IS SHARED IN "XFDLBPEL_XSD.XSL", "XFDLBPEL_WSDL.XSL" AND "XFDLBPEL_BPEL.XSL"!)
    *****************************************************************************************************************************
   -->
  <xsl:template name="program-activity-type">
    <xsl:param name="activity"/>
    <xsl:param name="server-may-be-missing" select="false()"/>
    <!-- ASSERT PRECONDITION -->
    <xsl:if test="not($activity)">
      <xsl:variable name="error-message" select="concat('PRECONDITION FAILED in template: name=&quot;program-activity-type&quot;', $eol, 'Parameter &quot;activity&quot; is not set')"/>
      <xsl:variable name="print-error-message" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.writeAssertion($error-message)"/>
    </xsl:if>
    <xsl:variable name="domain" select="//fdl:domain[1]"/>
    <xsl:variable name="system-group" select="//fdl:group[1]"/>
    <xsl:variable name="primary-system" select="//fdl:system[ @primarySystem ='yes' ][1]"/>
    <!-- Note that the following variables will contain appropriate values for "service invocation activities" (UPES) only -->
    <xsl:variable name="program-execution-unit" select="$activity/fdl:program/@programExecutionUnit"/>
    <xsl:variable name="server-name" select="substring-before($program-execution-unit, '.')"/>
    <xsl:variable name="related-system-name" select="substring-before(substring-after($program-execution-unit, concat($server-name, '.')), '.')"/>
    <xsl:variable name="server-and-system-name" select="concat($server-name, $related-system-name)"/>
    <xsl:choose>
      <!--
        CASE: "empty activity"
          A program activity is an "empty activity" if
          1) if the activity implementation is a program with the name FMCINTERNALNOOP
          2) the execution mode is chained
          3) the input and output data structures are the same
      -->
      <xsl:when test="$activity/fdl:program/@name='FMCINTERNALNOOP'
                and $activity/fdl:program/@synchronization='chained'
                and $activity/fdl:input/@type = $activity/fdl:output/@type">
        <xsl:value-of select="'empty activity'"/>
      </xsl:when>
      <!--
        CASE: "service invocation activity"
          A program activity is an "service invocation activity" if a user-defined program execution server
          (UPES) is specified (attribute "fdl:program/@programExecutionUnit").
      -->
      <xsl:when test="$activity/fdl:program/@programExecutionUnit
                  and (key('server-by-server-and-system-name', $server-and-system-name)[1]/fdl:userDefinedProgramExecutionServer or $server-may-be-missing = 'true')">
        <xsl:choose>
          <!-- CASE: User option "-un" (no UPES migration) is selected (cp. LI754 (Do not prepare process for reuse of UPES) -->
          <xsl:when test="'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
            <xsl:value-of select="'service invocation activity'"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="'empty activity'"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!--
        CASE: "staff activity"
          A program activity is a "staff activity" if the following conditions apply:
          1. Start type is "manual"
          2. Attribute "checkoutPossible" indicates that it can be checked out from the runtime database.
      -->
      <xsl:when test="$activity/fdl:join/@startType='manual'
                  and ( $activity/fdl:defaultSetting/@checkoutPossible='yes'
                       or (  not($activity/fdl:defaultSetting/@checkoutPossible='no')
                          and (   $activity/ancestor::fdl:process/fdl:activityDefaultSetting/@checkoutPossible='yes'
                              or (   'TRUE' = translate($LI1274_Support_inheritance_from_network_settings, $lower-case-characters, $upper-case-characters)
                                  and not($activity/ancestor::fdl:process/fdl:activityDefaultSetting/@checkoutPossible='no')
                                  and (  $primary-system/fdl:activityDefaultSetting/@checkoutPossible='yes'
                                      or (  not($primary-system/fdl:activityDefaultSetting/@checkoutPossible='no')
                                          and (   $system-group/fdl:activityDefaultSetting/@checkoutPossible='yes'
                                              or (  not($system-group/fdl:activityDefaultSetting/@checkoutPossible='no')
                                                  and $domain/fdl:activityDefaultSetting/@checkoutPossible='yes' ) ) ) ) ) ) ) )">
        <xsl:value-of select="'staff activity'"/>
      </xsl:when>
      <!--
        OTHERWISE: "unclassified activity"
          A program activity is an "unclassified activity" if none of the above rules apply.
          In this case assume a "staff activity".
      -->
      <xsl:otherwise>
        <xsl:value-of select="'assumed staff activity'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:transform>
