<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform xmlns:bpc="http://www.ibm.com/xmlns/prod/websphere/business-process/6.0.0/" xmlns:wpc="http://www.ibm.com/xmlns/prod/websphere/business-process/6.0.0/" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http:///extensionmodel.ecore" xmlns:extensionmodel="http:///extensionmodel.ecore" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:bpws="http://schemas.xmlsoap.org/ws/2004/03/business-process/" xmlns:xalan="http://xml.apache.org/xalan" xmlns:java="http://xml.apache.org/xalan/java" extension-element-prefixes="xalan java" exclude-result-prefixes="bpws" version="1.0">
  <!--
    *************************************************************************

      Workfile: BPCSRVR/ws/code/workflow.fdl2bpel/src/com/ibm/bpe/fdl2bpel/converter/v62/xfdl2bpel_bpelex.xsl, BPC.workflow, WBI70.BPCSRVR, of0950.12
      Last update: 09/01/14 03:55:50
      SCCS path, id: /family/botp/vc/13/4/8/5/s.57 1.8

    *************************************************************************
    BEGIN COPYRIGHT
    *************************************************************************

      IBM Confidential
      OCO Source Materials
      5655-FLW (C) Copyright IBM Corporation 2003, 2008.
      The source code for this program is not published or otherwise
      divested of its trade secrets, irrespective of what has been
      deposited with the U.S. Copyright Office.

    *************************************************************************
    END COPYRIGHT
  -->
  <!--
    *****************************************************************************************************************************
          Task:      Create a BPELEX file that represents graphical layout information of a BPEL diagram
    *****************************************************************************************************************************
          Author: Jens Grotrian (jgr@de.ibm.com)
    *****************************************************************************************************************************
   -->
  <xsl:strip-space elements="*"/>
  <xsl:output method="xml" indent="yes" xalan:indent-amount="5"/>
  <!--
    *****************************************************************************************************************************
         Configurable parameters
    *****************************************************************************************************************************
  -->
  <!-- "process-name": the name of the FDL process to be converted -->
  <xsl:param name="process-name"/>
  <!-- "create_SCA_UPES_IMPORTS_only" (cp. LI1195: Invocation of existing user-defined pgm execution servers in new BPEL processes) -->
  <xsl:param name="create_SCA_UPES_IMPORTS_only" select="'FALSE'"/>
  <!--************************************ -->
  <!--*        Global parameters             * -->
  <!--************************************ -->
  <!-- "end of line" character string -->
  <xsl:param name="eol" select="java:java.lang.System.getProperty('line.separator')"/>
  <!--
    *** Parameters also used in stylesheet "xfdl2bpel_bpel.xsl" ***
  -->
  <!-- "lower-case-characters": "aäbcdefghijklmnoöpqrstuüvwxyz" -->
  <xsl:param name="lower-case-characters" select="'aäbcdefghijklmnoöpqrstuüvwxyz'"/>
  <!-- "upper-case-characters": "AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ" -->
  <xsl:param name="upper-case-characters" select="'AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ'"/>
  <!-- "Execute-activities-contained-in-process": Display name used for a flow activity that contains the activities of a process -->
  <xsl:param name="Execute-activities-contained-in-process" select="'Execute activities contained in process'"/>
  <!-- "Execute-activities-contained-in-block": Display name used for a flow activity that contains the activities of a block -->
  <xsl:param name="Execute-activities-contained-in-block" select="'Execute activities contained in block'"/>
  <!--
    *****************************************************************************************************************************
    Initial template
    *****************************************************************************************************************************
  -->
  <xsl:template match="/">
    <xsl:if test="not( 'TRUE' = translate($create_SCA_UPES_IMPORTS_only, $lower-case-characters, $upper-case-characters) )">
      <xsl:apply-templates select="//bpws:process" mode="process"/>
    </xsl:if>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
    "process" template:

    Creates a "BPELEX" file for "bpws:process" (writes the result in a separate file with extension ".bpelex")
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:process[ @name = $process-name ]" mode="process">
    <xsl:variable name="href-value" select="concat($process-name, '.bpel#/')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensionmodel:ExtensionMap xmi:version="2.0" xmlns:xmi="http://www.omg.org/XMI" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:com.ibm.wbit.bpel.ui.uiextensionmodel="http:///com/ibm/wbit/bpel/ui/uiextensionmodel.ecore" xmlns:extensionmodel="http:///extensionmodel.ecore" namespace="http://com.ibm.wbit.bpel.ui/">
      <xsl:apply-templates select="bpws:invoke
                            | bpws:empty
                            | bpws:assign
                            | bpws:receive
                            | bpws:reply
                            | bpws:sequence
                            | bpws:flow
                            | bpws:while
                            | bpws:repeatUntil
                            | bpws:scope
                            | bpws:faultHandlers" mode="activity">
        <xsl:with-param name="href-value" select="$href-value"/>
        <xsl:with-param name="collapse-structured-activities" select="false()"/>
      </xsl:apply-templates>
      <extensions>
        <extendedObject href="{$href-value}"/>
        <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ProcessExtension" autoLayout="true"/>
      </extensions>
    </extensionmodel:ExtensionMap>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activity" template:

    Matches any BPEL "simple" activity without siblings wrt. its parent node
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:invoke | bpws:empty | bpws:assign | bpws:receive | bpws:reply" mode="activity">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activity')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "simple" activity which may have multiple siblings wrt. its parent node
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:invoke | bpws:empty | bpws:assign | bpws:receive | bpws:reply" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activities.', position() - 1)"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activity" template:

    Matches any BPEL "flow" activity
     - without siblings wrt. its parent node
         - may have multiple child nodes
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:flow" mode="activity">
    <xsl:param name="href-value"/>
    <xsl:param name="collapse-structured-activities" select="true()"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activity')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <xsl:choose>
        <xsl:when test="$collapse-structured-activities = 'true' and not( @wpc:displayName = $Execute-activities-contained-in-process or @wpc:displayName=$Execute-activities-contained-in-block )">
          <extensionObject collapsed="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:when>
        <xsl:otherwise>
          <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:otherwise>
      </xsl:choose>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activity" template:

    Matches any BPEL "structure" activity
     - without siblings wrt. its parent node
         - may have multiple child nodes
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:sequence" mode="activity">
    <xsl:param name="href-value"/>
    <xsl:param name="collapse-structured-activities" select="true()"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activity')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject implicit="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "flow" activity
     - which may have multiple siblings wrt. its parent node
         - may have multiple child nodes
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:flow" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:param name="collapse-structured-activities" select="true()"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activities.', position() - 1)"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <xsl:choose>
        <xsl:when test="$collapse-structured-activities = 'true' and not( @wpc:displayName = $Execute-activities-contained-in-process or @wpc:displayName=$Execute-activities-contained-in-block )">
          <extensionObject collapsed="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:when>
        <xsl:otherwise>
          <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:otherwise>
      </xsl:choose>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "structure" activity
     - which may have multiple siblings wrt. its parent node
         - may have multiple child nodes
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:sequence" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:param name="collapse-structured-activities" select="true()"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activities.', position() - 1)"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <xsl:choose>
        <xsl:when test="$collapse-structured-activities = 'true'">
          <extensionObject collapsed="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:when>
        <xsl:otherwise>
          <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
        </xsl:otherwise>
      </xsl:choose>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activity" template:

    Matches any BPEL "while" or "scope" activity
     - without siblings wrt. its parent node
         - may have at most a single child node
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:while | bpws:repeatUntil | bpws:scope" mode="activity">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activity')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activity">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "while" or "scope" activity
     - which may have multiple siblings wrt. its parent node
         - may have at most a single child node
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:while | bpws:repeatUntil | bpws:scope" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@activities.', position() - 1)"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @wpc:displayName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activity">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activity" template:

    Matches any BPEL "bpws:faultHandlers" activity where the parent has no siblings.
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:faultHandlers" mode="activity">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@faultHandlers')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;faultHandlers&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject collapsed="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:FaultHandlerExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:catch" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "bpws:faultHandlers" activity where the parent may have multiple siblings.
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:faultHandlers" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@faultHandlers')"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;faultHandlers&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject collapsed="true" xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:FaultHandlerExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:catch" mode="activities">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     "activities" template:

    Matches any BPEL "bpws:catch" activity
    *****************************************************************************************************************************
  -->
  <xsl:template match="bpws:catch" mode="activities">
    <xsl:param name="href-value"/>
    <xsl:variable name="current-href-value" select="concat($href-value, '/@catch.', position() - 1)"/>
    <xsl:value-of select="concat($eol, '&lt;!-- &quot;', @faultName, '&quot; --&gt;', $eol)" disable-output-escaping="yes"/>
    <extensions>
      <extendedObject href="{$current-href-value}"/>
      <extensionObject xsi:type="com.ibm.wbit.bpel.ui.uiextensionmodel:ActivityExtension"/>
    </extensions>
    <xsl:apply-templates select="bpws:invoke
                          | bpws:empty
                          | bpws:assign
                          | bpws:receive
                          | bpws:reply
                          | bpws:sequence
                          | bpws:flow
                          | bpws:while
                          | bpws:repeatUntil
                          | bpws:scope
                          | bpws:faultHandlers" mode="activity">
      <xsl:with-param name="href-value" select="$current-href-value"/>
    </xsl:apply-templates>
  </xsl:template>
</xsl:transform>
