<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:wsdls="http://schemas.xmlsoap.org/wsdl/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:redirect="http://xml.apache.org/xalan/redirect" xmlns:xalan="http://xml.apache.org/xalan" xmlns:java="http://xml.apache.org/xalan/java" extension-element-prefixes="redirect xalan java" version="1.0">
  <!--
    *************************************************************************

      Workfile: BPCSRVR/ws/code/workflow.fdl2bpel/src/com/ibm/bpe/fdl2bpel/converter/v62/xfdl2bpel_java.xsl, BPC.workflow, WBI70.BPCSRVR, of0950.12
      Last update: 09/01/14 03:56:30
      SCCS path, id: /family/botp/vc/13/4/8/5/s.58 1.4

    *************************************************************************
    BEGIN COPYRIGHT
    *************************************************************************

      IBM Confidential
      OCO Source Materials
      5655-FLW (C) Copyright IBM Corporation 2003, 2008.
      The source code for this program is not published or otherwise
      divested of its trade secrets, irrespective of what has been
      deposited with the U.S. Copyright Office.

    *************************************************************************
    END COPYRIGHT
  -->
  <!--
    *****************************************************************************************************************************
          Task:      Create Java binding classes that belong to an SCA module
    *****************************************************************************************************************************
          Authors:     Jens Grotrian (jgr@de.ibm.com)
    *****************************************************************************************************************************
   -->
  <xsl:strip-space elements="*"/>
  <xsl:output method="text" indent="no"/>
  <!--************************************ -->
  <!--*        Configurable parameters   * -->
  <!--************************************ -->
  <!-- "fdlfile": FDL file name -->
  <xsl:param name="fdlfile" select="''"/>
  <!-- "wsdlfile": path of target directory -->
  <xsl:param name="wsdlfile" select="''"/>
  <!-- "outputdir": path of target directory -->
  <xsl:param name="outputdir" select="''"/>
  <!-- "temp-Java-file-name-extension": temporary file name extensions for the UPES data binding classes (used before code page conversion) -->
  <xsl:param name="temp-Java-file-name-extension" select="'.jjj'"/>
  <!-- "variable-type-mode": Allowed values are "DATA" (default) or "INTERFACE" -->
  <xsl:param name="variable-type-mode" select="'DATA'"/>
  <!--************************** -->
  <!--*        Parameters         * -->
  <!--************************** -->
  <!-- "end of line" character string -->
  <xsl:param name="eol" select="java:java.lang.System.getProperty('line.separator')"/>
  <!-- "mqwfTargetNamespace": MQWF target namespace used for UPES message data binding -->
  <xsl:param name="mqwfTargetNamespace" select="'http://www.ibm.com/xmlns/prod/websphere/mqwf/schema/'"/>
  <!-- "mqwfTargetNamespacePrefix": MQWF target namespace prefix -->
  <xsl:param name="mqwfTargetNamespacePrefix" select="'mqwf:'"/>
  <!-- "wsdlTargetNamespacePrefix": WSDL target namespace prefix -->
  <xsl:param name="wsdlTargetNamespacePrefix" select="'wsdl1:'"/>
  <!--
    *** Parameters also used in stylesheet "xfdl2bpel_wsdl.xsl" ***
  -->
  <!-- "lower-case-characters": "aäbcdefghijklmnoöpqrstuüvwxyz" -->
  <xsl:param name="lower-case-characters" select="'aäbcdefghijklmnoöpqrstuüvwxyz'"/>
  <!-- "upper-case-characters": "AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ" -->
  <xsl:param name="upper-case-characters" select="'AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ'"/>
  <!-- "message-suffix": Suffix appended to "standard" message names  -->
  <xsl:param name="message-suffix" select="'_MSG'"/>
  <!-- "UPES-response-suffix": Suffix appended to UPES response messages  -->
  <xsl:param name="UPES-response-suffix" select="'_Response_UPES'"/>
  <!--
    *** Parameters also used in stylesheet "xfdl2bpel_bpel.xsl" ***
  -->
  <!-- "partner-link-suffix": Suffix appended to "partner link" names -->
  <xsl:param name="partner-link-suffix" select="'_PL'"/>
  <!-- "Error" Message severity 1 -->
  <xsl:param name="Error" select="1"/>
  <!-- "Warning" Message severity 2 -->
  <xsl:param name="Warning" select="2"/>
  <!-- "Information" Message severity 3 -->
  <xsl:param name="Information" select="3"/>
  <!--
    *** Parameters also used in stylesheet "xfdl2bpel_scdl.xsl" ***
  -->
  <!-- "data-binding-suffix": Suffix appended to data binding names -->
  <xsl:param name="data-binding-suffix" select="'_DataBindingImpl'"/>
  <!-- "data-binding-package-name": package name of a data binding Java classes -->
  <xsl:param name="data-binding-package-name" select="'com.ibm.workflow.sca.jms.data'"/>
  <!-- "data-binding-package-relative-path": relative path of data binding Java classes -->
  <xsl:param name="data-binding-package-path" select="'com\ibm\workflow\sca\jms\data\'"/>
  <!--
    *****************************************************************************************************************************
    Initial template
    *****************************************************************************************************************************
  -->
  <xsl:template match="/">
    <xsl:call-template name="create-UPES-data-bindings"/>
  </xsl:template>
  <!--
    *****************************************************************************************************************************
     Template "create-UPES-data-bindings"

    Creates the data bindings for UPES message types
    *****************************************************************************************************************************
  -->
  <xsl:template name="create-UPES-data-bindings">
    <xsl:variable name="messages" select="document($wsdlfile)/wsdls:definitions/wsdls:message"/>
    <xsl:variable name="UPES-output-message-suffix" select="concat($UPES-response-suffix, $message-suffix)"/>
    <!-- For each  UPES message which ends with '_Response_UPES_MSG' -->
    <xsl:for-each select="xalan:nodeset($messages)[ substring(@name, string-length(@name) - string-length($UPES-output-message-suffix) + 1) = $UPES-output-message-suffix ]">
      <xsl:variable name="className" select="concat(translate(substring(current()/@name, 1, 1), $lower-case-characters, $upper-case-characters), substring(current()/@name, 2))"/>
      <xsl:variable name="temp-Java-file-local-name" select="concat($className, $data-binding-suffix, $temp-Java-file-name-extension)"/>
      <xsl:variable name="Java-file-local-name" select="concat($className, $data-binding-suffix, '.java')"/>
      <xsl:variable name="Java-file-path-name" select="concat($outputdir, $data-binding-package-path, $temp-Java-file-local-name)"/>
      <xsl:variable name="addJavaFileName" select="java:com.ibm.bpe.fdl2bpel.converter.FileNameList.add($Java-file-path-name)"/>
      <xsl:variable name="message" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Information, 'writeOutput', $Java-file-local-name)"/>
      <xsl:variable name="message-type">
        <xsl:choose>
          <!-- CASE: interface-typed BPEL variables -->
          <xsl:when test="'INTERFACE' = translate($variable-type-mode, $lower-case-characters, $upper-case-characters)">
            <xsl:value-of select="substring(current()/wsdls:part/@type, string-length($mqwfTargetNamespacePrefix)+1)"/>
          </xsl:when>
          <!-- OTHERWISE: data-typed BPEL variables assumed -->
          <xsl:otherwise>
            <xsl:variable name="XMLSchema-elements" select="document($wsdlfile)/wsdls:definitions/wsdls:types/xsd:schema/xsd:element"/>
            <xsl:variable name="message-XMLSchema-element" select="substring(current()/wsdls:part/@element, string-length($wsdlTargetNamespacePrefix)+1)"/>
            <xsl:variable name="current-XMLSchema-element" select="xalan:nodeset($XMLSchema-elements)[ @name = $message-XMLSchema-element ]"/>
            <xsl:value-of select="substring($current-XMLSchema-element/xsd:complexType/xsd:sequence/xsd:element/@type, string-length($mqwfTargetNamespacePrefix)+1)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <!-- ASSERTION -->
      <xsl:if test="not($message-type)">
        <xsl:variable name="error-message" select="'ASSERTION FAILED in template &quot;create-UPES-data-bindings&quot; of stylesheet &quot;xfdl2bpel_java&quot;: variable &quot;message-type&quot; could not be resolved'"/>
        <xsl:value-of select="concat('*** ', java:com.ibm.bpe.fdl2bpel.converter.CEnv.writeAssertion($error-message), ' *** ')" disable-output-escaping="no"/>
      </xsl:if>
      <redirect:write select="$Java-file-path-name">
        <xsl:call-template name="create-UPES-data-binding">
          <xsl:with-param name="className" select="$className"/>
          <xsl:with-param name="message-type" select="$message-type"/>
        </xsl:call-template>
      </redirect:write>
    </xsl:for-each>
  </xsl:template>
  <!-- Generate DataBinding.java file -->
  <!--
    *****************************************************************************************************************************
     Template "create-UPES-data-binding"

    Creates a Java class file that binds a WSDL message type to a Java data type.
    *****************************************************************************************************************************
  -->
  <xsl:template name="create-UPES-data-binding">
    <xsl:param name="className"/>
    <xsl:param name="message-type"/>
    <xsl:text>/*</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text> * Generated by FDL2BPEL converter on </xsl:text>
    <xsl:value-of select="java:java.util.Date.new()"/>
    <xsl:value-of select="$eol"/>
    <xsl:text> * from FDL file </xsl:text>
    <xsl:value-of select="concat('&quot;', $fdlfile, '&quot;')"/>
    <xsl:value-of select="$eol"/>
    <xsl:text> *</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text> */</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>package </xsl:text>
    <xsl:value-of select="$data-binding-package-name"/>
    <xsl:text>;</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:value-of select="$eol"/>
    <xsl:text>public class </xsl:text>
    <xsl:value-of select="$className"/>
    <xsl:text>_DataBindingImpl extends WMQWFUpesDataBinding {</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:value-of select="$eol"/>
    <xsl:text>    private static final long serialVersionUID = 1L;</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:value-of select="$eol"/>
    <xsl:text>    public </xsl:text>
    <xsl:value-of select="$className"/>
    <xsl:text>_DataBindingImpl() {</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>        super();</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>        this.setDataType("</xsl:text>
    <xsl:value-of select="$message-type"/>
    <xsl:text>");</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>        this.setDataTypeNameSpace("</xsl:text>
    <xsl:value-of select="$mqwfTargetNamespace"/>
    <xsl:text>");</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>    }</xsl:text>
    <xsl:value-of select="$eol"/>
    <xsl:text>}</xsl:text>
    <xsl:value-of select="$eol"/>
  </xsl:template>
</xsl:transform>
