<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:wpc="http://www.ibm.com/xmlns/prod/websphere/business-process/6.0.0/" xmlns:bpws="http://schemas.xmlsoap.org/ws/2004/03/business-process/" xmlns:xalan="http://xml.apache.org/xalan" extension-element-prefixes="xalan" exclude-result-prefixes="bpws wpc" version="1.0">
  <!--
    *************************************************************************

      Workfile: BPCSRVR/ws/code/workflow.fdl2bpel/src/com/ibm/bpe/fdl2bpel/converter/v62/xfdl2bpel_opt_snippet.xsl, BPC.workflow, WBI70.BPCSRVR, of0950.12
      Last update: 09/01/14 03:57:09
      SCCS path, id: /family/botp/vc/13/7/8/8/s.64 1.4

    *************************************************************************
    BEGIN COPYRIGHT
    *************************************************************************

      IBM Confidential
      OCO Source Materials
      5655-FLW (C) Copyright IBM Corporation 2003, 2008.
      The source code for this program is not published or otherwise
      divested of its trade secrets, irrespective of what has been
      deposited with the U.S. Copyright Office.

    *************************************************************************
    END COPYRIGHT
  -->
  <!--
    *****************************************************************************************************************************
          Task:            Optimize BPEL file by merging sequential chains of Java snippets
        Prerequisite: This stylesheet requires a BPEL file as input that results from running FDL2BPEL
                                    without any optimization. Such BPEL file is characterized by wrapping the translation
                                    of each migrated "Program Activity", "ProcessActivity", or "Block" with a BPEL "Sequence".
                                    Each "Sequence" of this type contains exactly one "Invoke" or "Empty" or "flow" activity.
                                    This activity can be preceded or followed by one or mor Java Snippets.
                                    Multiple preceding Java snippets will be merged to a single Java snippet with name "Set up activity".
                  Accordingly. multiple following Java snippets will be merged to a single Java snippet with name
                                    "Shut down activity".
                                    This stylesheet does not work properly, if these assumptions are not satisfied by the
                                    input BPEL file.
    *****************************************************************************************************************************
          Author: Jens Grotrian (jgr@de.ibm.com)
    *****************************************************************************************************************************
   -->
  <xsl:output method="xml" indent="yes" xalan:indent-amount="5" cdata-section-elements="wpc:description wpc:documentation wpc:value bpws:from bpws:query wpc:javaCode"/>
  <!--************************************ -->
  <!--*        Configurable parameters   * -->
  <!--************************************ -->
  <!-- "bpelFileName": BPEL file name -->
  <xsl:param name="bpelFileName" select="'unknown BPEL file name'"/>
  <!--
    *****************************************************************************************************************************
    Initial template
    *****************************************************************************************************************************
  -->
  <xsl:template match="/">
    <xsl:apply-templates mode="copy"/>
  </xsl:template>
  <!-- Don't copy comment before a Java snippet, because optimization would misplace it -->
  <xsl:template match="comment()[ following-sibling::bpws:invoke[ wpc:script/wpc:javaCode ] and parent::bpws:sequence ]" mode="copy"/>
  <!-- Don't copy the Java snippets, because they may be candidates for a merging operation (see next template) -->
  <xsl:template match="bpws:sequence/bpws:invoke[ wpc:script/wpc:javaCode ]" mode="copy"/>
  <!-- Copy "Invoke" or "Parallel activities" or "Empty" node together with merged Java snippets -->
  <xsl:template match="bpws:sequence/bpws:invoke[ not(wpc:script/wpc:javaCode) ] | bpws:sequence/bpws:while | bpws:sequence/bpws:repeatUntil | bpws:sequence/bpws:flow | bpws:sequence/bpws:empty" mode="copy">
    <!-- Find Java snippets that precede the "Invoke" or "Parallel activities" or "Empty" node -->
    <xsl:variable name="preceding-snippets" select="preceding-sibling::node()[ wpc:script/wpc:javaCode ]"/>
    <xsl:if test="$preceding-snippets">
      <xsl:choose>
        <!-- CASE: More than one Java snippet found -->
        <xsl:when test="count($preceding-snippets) &gt; 1">
          <!-- Create a new Java snippet that merges the list of found Java snippets to a single one -->
          <bpws:invoke portType="wpc:null" partnerLink="null" operation="null" name="{$preceding-snippets[1]/@name}" wpc:displayName="Set up activity">
            <wpc:script>
              <wpc:javaCode>
                <xsl:for-each select="preceding-sibling::node()/wpc:script/wpc:javaCode/text()">
                  <xsl:value-of select="current()"/>
                </xsl:for-each>
              </wpc:javaCode>
            </wpc:script>
          </bpws:invoke>
        </xsl:when>
        <!-- OTHERWISE: One Java snippet found (no merging necessary) -->
        <xsl:otherwise>
          <!-- Copy the Java snippet with it's respective comments -->
          <xsl:copy-of select="preceding-sibling::node()[ wpc:script/wpc:javaCode ] | preceding-sibling::comment() | preceding-sibling::text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <!-- Copy the "Invoke" or "Parallel activities" or "Empty" node -->
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="copy"/>
      <xsl:apply-templates mode="copy"/>
    </xsl:copy>
    <!-- Find Java snippets that follow the "Invoke" or "Parallel activities" or "Empty" node -->
    <xsl:variable name="following-snippets" select="following-sibling::node()[ wpc:script/wpc:javaCode ]"/>
    <xsl:if test="$following-snippets">
      <xsl:choose>
        <!-- CASE: More than one Java snippet found -->
        <xsl:when test="count($following-snippets) &gt; 1">
          <!-- Create a new Java snippet that merges the list of found Java snippets to a single one -->
          <bpws:invoke portType="wpc:null" partnerLink="null" operation="null" name="{$following-snippets[1]/@name}" wpc:displayName="Shut down activity">
            <wpc:script>
              <wpc:javaCode>
                <xsl:for-each select="following-sibling::node()/wpc:script/wpc:javaCode/text()">
                  <xsl:value-of select="current()"/>
                </xsl:for-each>
              </wpc:javaCode>
            </wpc:script>
          </bpws:invoke>
        </xsl:when>
        <!-- OTHERWISE: One Java snippet found (no merging necessary) -->
        <xsl:otherwise>
          <!-- Copy the Java snippet with it's respective comments -->
          <xsl:copy-of select="following-sibling::node()[ wpc:script/wpc:javaCode ] | following-sibling::comment() | following-sibling::text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  <!-- Copy all remaining attributes and nodes that do not match the above rules -->
  <xsl:template match="@* | node()" mode="copy">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="copy"/>
      <xsl:apply-templates mode="copy"/>
    </xsl:copy>
  </xsl:template>
</xsl:transform>
