<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fdl="http://www.ibm.com/workflow/2003" xmlns:xalan="http://xml.apache.org/xalan" xmlns:java="http://xml.apache.org/xalan/java" extension-element-prefixes="xalan java" exclude-result-prefixes="fdl" xmlns="http://www.w3.org/2001/XMLSchema" version="1.0">
  <!--
    *************************************************************************

      Workfile: BPCSRVR/ws/code/workflow.fdl2bpel/src/com/ibm/bpe/fdl2bpel/converter/v62/xfdl2bpel_xsd.xsl, BPC.workflow, WBI70.BPCSRVR, of0950.12
      Last update: 09/01/14 03:59:37
      SCCS path, id: /family/botp/vc/13/4/8/5/s.61 1.13

    *************************************************************************
    BEGIN COPYRIGHT
    *************************************************************************

      IBM Confidential
      OCO Source Materials
      5655-FLW (C) Copyright IBM Corporation 2003, 2008.
      The source code for this program is not published or otherwise
      divested of its trade secrets, irrespective of what has been
      deposited with the U.S. Copyright Office.

    *************************************************************************
    END COPYRIGHT
  -->
  <!--
    *****************************************************************************************************************************
          Task:     Create a XMLSchema definition file for the data objects used in BPEL processes
    *****************************************************************************************************************************
          Author: Jens Grotrian (jgr@de.ibm.com)
    *****************************************************************************************************************************
   -->
  <xsl:strip-space elements="*"/>
  <xsl:output method="xml" indent="yes" xalan:indent-amount="5"/>
  <!--************************************ -->
  <!--*      Configurable parameters     * -->
  <!--************************************ -->
  <!-- "fdlFileName": FDL file name -->
  <xsl:param name="fdlFileName" select="'unknown FDL file name'"/>
  <!-- "createPredefinedDataMembers": Create the predefined MQWF data structure members -->
  <xsl:param name="createPredefinedDataMembers" select="'TRUE'"/>
  <!-- "createUPESInvocations" (cp. LI754: Do not prepare process for reuse of UPES ) -->
  <xsl:param name="createUPESInvocations" select="'TRUE'"/>
  <!-- "create_SCA_UPES_IMPORTS_only" (cp. LI1195: Invocation of existing user-defined pgm execution servers in new BPEL processes) -->
  <xsl:param name="create_SCA_UPES_IMPORTS_only" select="'FALSE'"/>
  <!-- "LI1274_Support_inheritance_from_network_settings" -->
  <xsl:param name="LI1274_Support_inheritance_from_network_settings" select="'FALSE'"/>
  <!--************************** -->
  <!--*        Parameters         * -->
  <!--************************** -->
  <!-- "lower-case-characters": "aäbcdefghijklmnoöpqrstuüvwxyz" -->
  <xsl:param name="lower-case-characters" select="'aäbcdefghijklmnoöpqrstuüvwxyz'"/>
  <!-- "Upper-case-characters": "AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ" -->
  <xsl:param name="upper-case-characters" select="'AÄBCDEFGHIJKLMNOÖPQRSTUÜVWXYZ'"/>
  <!-- "message-type-suffix": Suffix appended to message type names -->
  <xsl:param name="message-type-suffix" select="'_MT'"/>
  <!-- "UPES-input-message-type-suffix": Suffix appended to UPES input message type names -->
  <xsl:param name="UPES-input-message-type-suffix" select="'_UPES_IN_MT'"/>
  <!-- "UPES-output-message-type-suffix": Suffix appended to UPES output message type names -->
  <xsl:param name="UPES-output-message-type-suffix" select="'_UPES_OUT_MT'"/>
  <!-- "global-container-message-type-suffix": Suffix appended to message type names that specify a "global container" -->
  <xsl:param name="global-container-message-type-suffix" select="'_GC_MT'"/>
  <!-- "Error" Message severity 1 -->
  <xsl:param name="Error" select="1"/>
  <!-- "Warning" Message severity 2 -->
  <xsl:param name="Warning" select="2"/>
  <!-- "Information" Message severity 3 -->
  <xsl:param name="Information" select="3"/>
  <!--************************************ -->
  <!--*        Global parameters             * -->
  <!--************************************ -->
  <!-- "end of line" character string -->
  <xsl:param name="eol" select="java:java.lang.System.getProperty('line.separator')"/>
  <!--************************** -->
  <!--*        Keys                   * -->
  <!--************************** -->
  <!--
    Key "UPES-activity-receiving-data-structure" represents a "mapping" between
    data structure names (as "key objects") and program activities (as "value objects")
  -->
  <xsl:key name="UPES-activity-receiving-data-structure" match="fdl:programActivity[ fdl:program/@programExecutionUnit ]" use="fdl:input/@type"/>
  <!--
    Key "UPES-activity-sending-data-structure" represents a "mapping" between
    data structure names (as "key objects") and program activities (as "value objects")
  -->
  <xsl:key name="UPES-activity-sending-data-structure" match="fdl:programActivity[ fdl:program/@programExecutionUnit ]" use="fdl:output/@type"/>
  <!--
    Key "data-structure-by-name" represents a "mapping" between  data structure type names (as "key objects") and data structure definitions (as "value objects")
  -->
  <xsl:key name="data-structure-by-name" match="fdl:structure" use="@name"/>
  <!--
    Key "process-by-name" represents a "mapping" between  process names (as "key objects") and process definitions (as "value objects")
  -->
  <xsl:key name="process-by-name" match="fdl:definitions/fdl:process
                                                                    | fdl:definitions/fdl:create/fdl:process
                                                                    | fdl:definitions/fdl:update/fdl:process
                                                                    | fdl:definitions/fdl:replace/fdl:process" use="@name"/>
  <!--
    Key "program-by-name" represents a "mapping" between  program names (as "key objects") and program definitions (as "value objects")
  -->
  <xsl:key name="program-by-name" match="fdl:definitions/fdl:program
                                                                    | fdl:definitions/fdl:create/fdl:program
                                                                    | fdl:definitions/fdl:update/fdl:program
                                                                    | fdl:definitions/fdl:replace/fdl:program" use="@name"/>
  <!--
    Key "server-by-server-and-system-name" represents a "mapping" between  the concatenated names of server and related system (as "key objects") and server definitions (as "value objects")
  -->
  <xsl:key name="server-by-server-and-system-name" match="fdl:definitions/fdl:server
                                                                    | fdl:definitions/fdl:create/fdl:server
                                                                    | fdl:definitions/fdl:update/fdl:server
                                                                    | fdl:definitions/fdl:replace/fdl:server" use="concat(@name, fdl:relatedSystem/@name)"/>
  <!--************* -->
  <!-- Templates   -->
  <!--************* -->
  <!--
    *****************
    Initial template
    *****************
  -->
  <xsl:template match="/">
    <!--
        "ERROR-CHECK 1": Checking the prerequisite for option "Do not create predefined data members"
        -->
    <xsl:variable name="error-check1-ok">
      <xsl:choose>
        <!-- CASE: "Create predefined data members" requested -->
        <xsl:when test="'TRUE' = translate($createPredefinedDataMembers, $lower-case-characters, $upper-case-characters)">
          <xsl:value-of select="'TRUE'"/>
        </xsl:when>
        <!-- OTHERWISE: Check whether predefined data members are used in the model -->
        <xsl:otherwise>
          <xsl:choose>
            <!-- CASE: Predefined data members are used in the model: Issue error message and stop migration -->
            <xsl:when test="//fdl:dataStructureMemberName/fdl:nameComponent[1]/@name != '_STRUCT'">
              <xsl:for-each select="//fdl:dataStructureMemberName[ fdl:nameComponent[1]/@name != '_STRUCT' ]">
                <!-- Error message: CWWBMnnnnE: Migration is not possible without the creation of predefined data members, because a predefined data member is used in your FDL file. -->
                <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'PredefinedDataMemberInUse', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:fdlLoc/@column)"/>
              </xsl:for-each>
              <xsl:value-of select="'FALSE'"/>
            </xsl:when>
            <!-- CASE: staff taken from predefined container member at runtime -->
            <xsl:when test="//fdl:process/fdl:staff/fdl:dataFromInputContainer or //fdl:programActivity/fdl:staff/fdl:staffDefinedInInputContainer or //fdl:processActivity/fdl:staff/fdl:staffDefinedInInputContainer">
              <xsl:for-each select="//fdl:process[ fdl:staff/fdl:dataFromInputContainer ] | //fdl:programActivity[ fdl:staff/fdl:staffDefinedInInputContainer ] | //fdl:processActivity[ fdl:staff/fdl:staffDefinedInInputContainer ]">
                <!-- Error message: CWWBMnnnnE: Migration is not possible without the creation of predefined data members, because a predefined data member is used in your FDL file. -->
                <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'PredefinedDataMemberInUse', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:fdlLoc/@column)"/>
              </xsl:for-each>
              <xsl:value-of select="'FALSE'"/>
            </xsl:when>
            <!-- CASE: notification taken from predefined container member at runtime -->
            <xsl:when test="//fdl:programActivity[ fdl:notification/@definedInInputContainer = 'yes' ]">
              <xsl:variable name="staff-activity-with-notification-taken-from-predefined-container-member-at-runtime">
                <xsl:for-each select="//fdl:programActivity[ fdl:notification/@definedInInputContainer = 'yes' ]">
                  <xsl:variable name="program-activity-type">
                    <xsl:call-template name="program-activity-type">
                      <xsl:with-param name="activity" select="current()"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:if test="$program-activity-type = 'staff activity' or $program-activity-type = 'assumed staff activity'">
                    <!-- Error message: CWWBMnnnnE: Migration is not possible without the creation of predefined data members, because a predefined data member is used in your FDL file. -->
                    <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'PredefinedDataMemberInUse', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:fdlLoc/@column)"/>
                    <xsl:value-of select="'FALSE'"/>
                  </xsl:if>
                </xsl:for-each>
              </xsl:variable>
              <xsl:choose>
                <xsl:when test="contains($staff-activity-with-notification-taken-from-predefined-container-member-at-runtime, 'FALSE')">
                  <xsl:value-of select="'FALSE'"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="'TRUE'"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <!-- CASE: priority taken from predefined container member at runtime -->
            <xsl:when test="//fdl:programActivity[ @priorityDefinedInInputContainer='yes' ]">
              <xsl:variable name="staff-activity-with-priority-taken-from-predefined-container-member-at-runtime">
                <xsl:for-each select="//fdl:programActivity[ @priorityDefinedInInputContainer='yes' ]">
                  <xsl:variable name="program-activity-type">
                    <xsl:call-template name="program-activity-type">
                      <xsl:with-param name="activity" select="current()"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:if test="$program-activity-type = 'staff activity' or $program-activity-type = 'assumed staff activity'">
                    <!-- Error message: CWWBMnnnnE: Migration is not possible without the creation of predefined data members, because a predefined data member is used in your FDL file. -->
                    <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'PredefinedDataMemberInUse', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:fdlLoc/@column)"/>
                    <xsl:value-of select="'FALSE'"/>
                  </xsl:if>
                </xsl:for-each>
              </xsl:variable>
              <xsl:choose>
                <xsl:when test="contains($staff-activity-with-priority-taken-from-predefined-container-member-at-runtime, 'FALSE')">
                  <xsl:value-of select="'FALSE'"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="'TRUE'"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <!-- OTHERWISE: Check is ok -->
            <xsl:otherwise>
              <xsl:value-of select="'TRUE'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
        "ERROR-CHECK 2": Checking for missing subprocess definitions
        -->
    <xsl:variable name="error-check2-ok">
      <xsl:choose>
        <!-- CASE: Usage of an undefined subprocess found -->
        <xsl:when test="//fdl:processActivity[ fdl:process/@name and not(key('process-by-name', fdl:process/@name)) ]">
          <xsl:for-each select="//fdl:processActivity[ fdl:process/@name and not(key('process-by-name', fdl:process/@name)) ]">
            <!--
                Error message: CWWBMnnnnE:
                Migration is not possible: The required process defintion nnn is referred to at activity nnn, but it is not defined in the FDL input file.
            -->
            <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'MissingProcessDefinition', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:process/@name, current()/@displayName)"/>
          </xsl:for-each>
          <xsl:value-of select="'FALSE'"/>
        </xsl:when>
        <!-- OTHERWISE: No undefined subprocess found -->
        <xsl:otherwise>
          <xsl:value-of select="'TRUE'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
        "ERROR-CHECK 3": Checking for any type mismatch between process activity and called subprocess
        -->
    <xsl:variable name="error-check3-ok">
      <xsl:choose>
        <!-- CASE: Process activity found -->
        <xsl:when test="//fdl:processActivity">
          <xsl:variable name="check-result">
            <xsl:for-each select="//fdl:processActivity">
              <!-- Check for type mismatch between process activity and called subprocess -->
              <xsl:if test="/fdl:definitions/fdl:process[ @name = current()/fdl:process/@name and ( fdl:input/@type != current()/fdl:input/@type or fdl:output/@type != current()/fdl:output/@type ) ]">
                <!--
                    Error message: CWWBMnnnnE:
                    Migration is not possible, because the specified input/output data types for process activity "nnn" are different from the input/output data types
                    expected by the referenced subprocess "nnn").
                -->
                <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'SubprocessCalledWithWrongDataType', $fdlFileName, current()/fdl:fdlLoc/@line, current()/@name, current()/fdl:process/@name)"/>
                <xsl:value-of select="'FALSE'"/>
              </xsl:if>
            </xsl:for-each>
          </xsl:variable>
          <xsl:choose>
            <!-- CASE: Type mismatch found -->
            <xsl:when test="contains(translate($check-result, $lower-case-characters, $upper-case-characters), 'FALSE')">
              <xsl:value-of select="'FALSE'"/>
            </xsl:when>
            <!-- OTHERWISE: No type mismatch found -->
            <xsl:otherwise>
              <xsl:value-of select="'TRUE'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <!-- OTHERWISE: No process activity found -->
        <xsl:otherwise>
          <xsl:value-of select="'TRUE'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
        "ERROR-CHECK 4": Checking for missing data structure definitions
        -->
    <xsl:variable name="error-check4-ok">
      <xsl:choose>
        <!-- CASE: Usage of an undefined data structure found -->
        <xsl:when test="//fdl:input[ not(key('data-structure-by-name', @type)) ] or //fdl:output[ not(key('data-structure-by-name', @type)) ]">
          <xsl:for-each select="//fdl:input[ not(key('data-structure-by-name', @type)) ] | //fdl:output[ not(key('data-structure-by-name', @type)) ]">
            <!--
                  Error message: CWWBMnnnnE:
                  Migration is not possible, because the specified input/output data type of nnn is not defined in the FDL.
              -->
            <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'MissingDataStructure', $fdlFileName, current()/../fdl:fdlLoc/@line, current()/@type, current()/../@displayName)"/>
          </xsl:for-each>
          <xsl:value-of select="'FALSE'"/>
        </xsl:when>
        <!-- OTHERWISE: No undefined data structure found -->
        <xsl:otherwise>
          <xsl:value-of select="'TRUE'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
        "ERROR-CHECK 5": Checking for missing program definitions
        -->
    <xsl:variable name="error-check5-ok">
      <xsl:choose>
        <!-- CASE: Usage of an undefined program found -->
        <xsl:when test="//fdl:programActivity[ fdl:program/@name and not(key('program-by-name', fdl:program/@name)) ]">
          <xsl:for-each select="//fdl:programActivity[ fdl:program/@name and not(key('program-by-name', fdl:program/@name)) ]">
            <!--
                  Error message: CWWBMnnnnE:
                  Migration is not possible: The program "{2}" is referred to at activity "{3}", but it is not defined in the FDL input file.
              -->
            <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'unknownProgramName', $fdlFileName, current()/fdl:fdlLoc/@line, current()/fdl:program/@name, current()/@displayName)"/>
          </xsl:for-each>
          <xsl:value-of select="'FALSE'"/>
        </xsl:when>
        <!-- OTHERWISE: No undefined data structure found -->
        <xsl:otherwise>
          <xsl:value-of select="'TRUE'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
        "ERROR-CHECK 6": Checking for missing UPES definitions
        -->
    <xsl:variable name="error-check6-ok">
      <!-- User option "-un" (no UPES migration) is selected (cp. LI754 (Do not prepare process for reuse of UPES) -->
      <xsl:if test="'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
        <xsl:choose>
          <!-- CASE: Program activity with program execution unit found -->
          <xsl:when test="//fdl:programActivity[ fdl:program/@programExecutionUnit ]">
            <xsl:variable name="check-result">
              <xsl:for-each select="//fdl:programActivity[ fdl:program/@programExecutionUnit ]">
                <xsl:variable name="assumed-program-activity-type">
                  <xsl:call-template name="program-activity-type">
                    <xsl:with-param name="activity" select="current()"/>
                    <xsl:with-param name="server-may-be-missing" select="true()"/>
                  </xsl:call-template>
                </xsl:variable>
                <xsl:if test="$assumed-program-activity-type = 'service invocation activity'">
                  <xsl:variable name="program-execution-unit" select="current()/fdl:program/@programExecutionUnit"/>
                  <xsl:variable name="server-name" select="substring-before($program-execution-unit, '.')"/>
                  <xsl:variable name="related-system-name" select="substring-before(substring-after($program-execution-unit, concat($server-name, '.')), '.')"/>
                  <xsl:variable name="server-and-system-name" select="concat($server-name, $related-system-name)"/>
                  <!-- Check for type mismatch between process activity and called subprocess -->
                  <xsl:if test="not(key('server-by-server-and-system-name', $server-and-system-name))">
                    <!--
                      Error message: CWWBMnnnnE:
                      Migration is not possible: The user-defined program execution server (UPES) "{2}" is referred to at activity "{3}", but it is not defined in the FDL input file.
                    -->
                    <xsl:variable name="error" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.write($Error, 'MissingUPES', $fdlFileName, current()/fdl:fdlLoc/@line, $program-execution-unit, current()/@displayName)"/>
                    <xsl:value-of select="'FALSE'"/>
                  </xsl:if>
                </xsl:if>
              </xsl:for-each>
            </xsl:variable>
            <xsl:choose>
              <!-- CASE: Missing UPES found -->
              <xsl:when test="contains(translate($check-result, $lower-case-characters, $upper-case-characters), 'FALSE')">
                <xsl:value-of select="'FALSE'"/>
              </xsl:when>
              <!-- OTHERWISE: No missing UPES found -->
              <xsl:otherwise>
                <xsl:value-of select="'TRUE'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <!-- OTHERWISE: No UPES activity found -->
          <xsl:otherwise>
            <xsl:value-of select="'TRUE'"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:variable>
    <!--
      Create the XMLSchema definitions provided that all plausibility checks passed successfully
    -->
    <xsl:if test="'TRUE' = translate($error-check1-ok, $lower-case-characters, $upper-case-characters)
              and 'TRUE' = translate($error-check2-ok, $lower-case-characters, $upper-case-characters)
              and 'TRUE' = translate($error-check3-ok, $lower-case-characters, $upper-case-characters)
              and 'TRUE' = translate($error-check4-ok, $lower-case-characters, $upper-case-characters)
              and 'TRUE' = translate($error-check5-ok, $lower-case-characters, $upper-case-characters)
              and ('FALSE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters) or 'TRUE' = translate($error-check6-ok, $lower-case-characters, $upper-case-characters))">
      <xsl:apply-templates mode="xsd-definitions"/>
    </xsl:if>
  </xsl:template>
  <!--
    *************************************************************
    Translate "fdl:definitions" into XMLSchema definitions
    *************************************************************
  -->
  <xsl:template match="fdl:definitions" mode="xsd-definitions">
    <schema elementFormDefault="unqualified" attributeFormDefault="qualified" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:mqwf="http://www.ibm.com/xmlns/prod/websphere/mqwf/schema/" targetNamespace="http://www.ibm.com/xmlns/prod/websphere/mqwf/schema/">
      <!-- Create user-defined data structure and message types -->
      <xsl:choose>
        <!--
          CASE: User option "-u" (reuse UPES only) was selected (cp. LI 1195: Invocation of existing user-defined pgm execution servers in new BPEL processes)
          Translate only such data structures that are needed for SCA Imports for UPES.
        -->
        <xsl:when test="'TRUE' = translate($create_SCA_UPES_IMPORTS_only, $lower-case-characters, $upper-case-characters)">
          <xsl:apply-templates select="fdl:structure | fdl:create/fdl:structure | fdl:update/fdl:structure | fdl:replace/fdl:structure" mode="xsd-definitions-for-UPES-only"/>
        </xsl:when>
        <!-- OTHERWISE: Translate all data structures -->
        <xsl:otherwise>
          <xsl:apply-templates select="fdl:structure | fdl:create/fdl:structure | fdl:update/fdl:structure | fdl:replace/fdl:structure" mode="xsd-definitions"/>
        </xsl:otherwise>
      </xsl:choose>
      <!-- Create data structure types required for "UPES" activities if user option "-un" (no UPES migration) is not selected (cp. LI754 (Do not prepare process for reuse of UPES) -->
      <xsl:if test="'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
        <xsl:call-template name="create-UPES-data-structure-types"/>
      </xsl:if>
    </schema>
  </xsl:template>
  <!--
    ******************************************************************
    Translate "fdl:structure" into XMLSchema "complexType"
    ******************************************************************
  -->
  <xsl:template match="fdl:structure" mode="xsd-definitions-for-UPES-only">
    <xsl:variable name="structure-contributes-to-UPES-activity">
      <xsl:call-template name="structure-contributes-to-UPES-activity">
        <xsl:with-param name="structure" select="current()"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:if test="contains($structure-contributes-to-UPES-activity, 'true')">
      <xsl:apply-templates select="current()" mode="xsd-definitions"/>
    </xsl:if>
  </xsl:template>
  <!--
    ****************************************************************************************************************
    Template "structure-contributes-to-UPES-activity":

    Determines whether the current structure contributes directly or indirectly
    to the input or output of a UPES activity
    ****************************************************************************************************************
   -->
  <xsl:template name="structure-contributes-to-UPES-activity">
    <xsl:param name="structure"/>
    <!-- ASSERT PRECONDITION (parameter "from-activity-name") -->
    <xsl:if test="not($structure)">
      <xsl:variable name="error-message" select="'PRECONDITION FAILED in template &quot;structure-contributes-to-UPES-activity&quot;: parameter &quot;structure&quot; not set'"/>
      <xsl:value-of select="concat($eol, '&lt;!-- *** ', java:com.ibm.bpe.fdl2bpel.converter.CEnv.writeAssertion($error-message), ' *** --&gt;', $eol)" disable-output-escaping="yes"/>
    </xsl:if>
    <xsl:choose>
      <!-- CASE: Structure belongs to the predefined members -->
      <xsl:when test="$structure/@name = '_ACTIVITY_INFO' or $structure/@name = '_PROCESS_INFO'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <!-- CASE: Structure is input of a UPES activity -->
      <xsl:when test="key('UPES-activity-receiving-data-structure', $structure/@name)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <!-- CASE: Structure is output of a UPES activity -->
      <xsl:when test="key('UPES-activity-sending-data-structure', $structure/@name)">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- CASE: Structure member of another data structure -->
        <xsl:for-each select="//fdl:structure[ fdl:member/@type = $structure/@name ]">
          <!-- Check whether any parent structure is input or output of a UPES activity -->
          <xsl:call-template name="structure-contributes-to-UPES-activity">
            <xsl:with-param name="structure" select="current()"/>
          </xsl:call-template>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
    ******************************************************************
    Translate "fdl:structure" into XMLSchema "complexType"
    ******************************************************************
  -->
  <xsl:template match="fdl:structure" mode="xsd-definitions">
    <!--
      Create complex type without "predefined data members".
    -->
    <complexType name="{@name}">
      <xsl:if test="*">
        <xsl:element name="sequence">
          <xsl:apply-templates select="fdl:member" mode="xsd-definitions"/>
        </xsl:element>
      </xsl:if>
    </complexType>
    <!--
      Create "complex type" definitions related to the input/output of a process, activity, or program.
    -->
    <xsl:if test="//*[ ( local-name() = 'process'
              or local-name() = 'programActivity'
              or local-name() = 'processActivity'
              or local-name() = 'block'
              or local-name() = 'program' )
              and (  fdl:input/@type = current()/@name or  fdl:output/@type = current()/@name ) ]">
      <!--
        Create  "complex type" definition that includes the "predefined data members", if it is referenced by some process, activity, or program.
      -->
      <xsl:if test="'TRUE' = translate($createPredefinedDataMembers, $lower-case-characters, $upper-case-characters)">
        <complexType name="{@name}{$message-type-suffix}">
          <sequence>
            <element type="xsd:long" minOccurs="0" name="_RC"/>
            <element type="xsd:string" minOccurs="0" name="_PROCESS"/>
            <element type="xsd:string" minOccurs="0" name="_PROCESS_MODEL"/>
            <element type="xsd:string" minOccurs="0" name="_ACTIVITY"/>
            <element type="mqwf:_PROCESS_INFO" name="_PROCESS_INFO"/>
            <element type="mqwf:_ACTIVITY_INFO" name="_ACTIVITY_INFO"/>
            <element type="mqwf:{@name}" name="_STRUCT"/>
          </sequence>
        </complexType>
      </xsl:if>
      <!--
        Create for each combination of input/output type a UPES input message type if we have an "UPES" activity
        provided that user option "-un" (no UPES migration) is not selected (cp. LI754 (Do not prepare process for reuse of UPES).
        (You have to consider "combination of input/output types", because the output type determines the type of element "MessageDefaultContainer"!)
       -->
      <xsl:if test="'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
        <xsl:variable name="current-data-structure-name" select="@name"/>
        <xsl:for-each select="key('UPES-activity-receiving-data-structure', $current-data-structure-name)">
          <xsl:variable name="current-UPES-activity" select="current()"/>
          <xsl:for-each select="ancestor::fdl:definitions/fdl:structure[ @name = $current-UPES-activity/fdl:output/@type ]
                      | ancestor::fdl:definitions/fdl:create/fdl:structure[ @name = $current-UPES-activity/fdl:output/@type ]
                      | ancestor::fdl:definitions/fdl:update/fdl:structure[ @name = $current-UPES-activity/fdl:output/@type ]
                      | ancestor::fdl:definitions/fdl:replace/fdl:structure[ @name = $current-UPES-activity/fdl:output/@type ]">
            <xsl:if test="not(current()/@name = $current-UPES-activity/preceding::fdl:programActivity[ fdl:input/@type = $current-data-structure-name and fdl:program/@programExecutionUnit ]/fdl:output/@type)">
              <xsl:choose>
                <!-- CASE: data structures of input and output are the same (we can use a simple type name) -->
                <xsl:when test="$current-data-structure-name = current()/@name">
                  <complexType name="{$current-data-structure-name}{$UPES-input-message-type-suffix}">
                    <sequence>
                      <element type="mqwf:_MESSAGE_CONTEXT" name="MessageContext"/>
                      <element type="mqwf:{$current-data-structure-name}" name="MessageContainer"/>
                      <element type="mqwf:{current()/@name}" name="MessageDefaultContainer"/>
                    </sequence>
                  </complexType>
                </xsl:when>
                <!--
                  OTHERWISE: different data structures of input and output (we need the names of input
                              and output data structures in order to prevent type name clashes)
                -->
                <xsl:otherwise>
                  <complexType name="{$current-data-structure-name}_{current()/@name}{$UPES-input-message-type-suffix}">
                    <sequence>
                      <element type="mqwf:_MESSAGE_CONTEXT" name="MessageContext"/>
                      <element type="mqwf:{$current-data-structure-name}" name="MessageContainer"/>
                      <element type="mqwf:{current()/@name}" name="MessageDefaultContainer"/>
                    </sequence>
                  </complexType>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
        <!--
          Create a single UPES output message type if we have an "UPES" activity.
         -->
        <xsl:if test="//fdl:programActivity[ fdl:output/@type = current()/@name and fdl:program/@programExecutionUnit ]">
          <complexType name="{@name}{$UPES-output-message-type-suffix}">
            <sequence>
              <element type="mqwf:_MESSAGE_CONTEXT" name="MessageContext"/>
              <element type="mqwf:{@name}" name="MessageContainer"/>
            </sequence>
          </complexType>
        </xsl:if>
      </xsl:if>
    </xsl:if>
    <!--
      Create another "complex type", if the FDL structure is referenced as a "global container" by some process.
    -->
    <xsl:if test="//fdl:process/fdl:globalContainer[ @type= current()/@name ]">
      <!-- Check whether parameter "createPredefinedDataMembers" is set to "true" -->
      <xsl:if test="'TRUE' = translate($createPredefinedDataMembers, $lower-case-characters, $upper-case-characters)">
        <!--
            Create another "complex type" definition with a single "_STRUCT" element.
        -->
        <complexType name="{@name}{$global-container-message-type-suffix}">
          <sequence>
            <element type="mqwf:{@name}" name="_STRUCT"/>
          </sequence>
        </complexType>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <!--
    ***********************************************************
    Translate "fdl:member" into XMLSchema "element"
    ***********************************************************
  -->
  <xsl:template match="fdl:member" mode="xsd-definitions">
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="'STRING' = translate(@type, $lower-case-characters, $upper-case-characters)">
          <xsl:text>xsd:string</xsl:text>
        </xsl:when>
        <xsl:when test="'LONG' = translate(@type, $lower-case-characters, $upper-case-characters)">
          <xsl:text>xsd:long</xsl:text>
        </xsl:when>
        <xsl:when test="'FLOAT' = translate(@type, $lower-case-characters, $upper-case-characters)">
          <xsl:text>xsd:double</xsl:text>
        </xsl:when>
        <xsl:when test="'BINARY' = translate(@type, $lower-case-characters, $upper-case-characters)">
          <xsl:text>xsd:hexBinary</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>mqwf:</xsl:text>
          <xsl:value-of select="@type"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="@cardinality">
        <element maxOccurs="{@cardinality}" type="{$type}" minOccurs="0" name="{@name}"/>
      </xsl:when>
      <xsl:otherwise>
        <element type="{$type}" minOccurs="0" name="{@name}"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
    *************************************************************
    Create "UPES" data structure types
    *************************************************************
  -->
  <xsl:template name="create-UPES-data-structure-types">
    <!-- Create data structure "_MESSAGE_CONTEXT" if there is an "UPES" activity and user option "-un" (no UPES migration) is not selected -->
    <xsl:if test="//fdl:programActivity/fdl:program/@programExecutionUnit and 'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
      <complexType name="_MESSAGE_CONTEXT">
        <sequence>
          <element type="xsd:long" minOccurs="0" name="_RC"/>
          <element type="xsd:string" minOccurs="0" name="_PROCESS"/>
          <element type="xsd:string" minOccurs="0" name="_PROCESS_MODEL"/>
          <element type="xsd:string" minOccurs="0" name="_ACTIVITY"/>
          <element type="mqwf:_PROCESS_INFO" minOccurs="0" name="_PROCESS_INFO"/>
          <element type="mqwf:_ACTIVITY_INFO" minOccurs="0" name="_ACTIVITY_INFO"/>
          <element type="xsd:string" minOccurs="0" name="ResponseRequired"/>
          <element type="xsd:string" minOccurs="0" name="UserContext"/>
          <element type="xsd:string" minOccurs="0" name="KeepName"/>
          <element type="xsd:string" minOccurs="0" name="ActImplCorrelID"/>
          <element type="xsd:string" minOccurs="0" name="Starter"/>
          <element type="xsd:string" minOccurs="0" name="ProgramName"/>
          <element type="xsd:string" minOccurs="0" name="MessageType"/>
          <element type="xsd:string" minOccurs="0" name="Origin"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstID"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstParentName"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstTopLevelName"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstDescription"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstState"/>
          <element type="xsd:string" minOccurs="0" name="LastStateChangeTime"/>
          <element type="xsd:string" minOccurs="0" name="LastModificationTime"/>
          <element type="xsd:string" minOccurs="0" name="ProcTemplID"/>
          <element type="xsd:string" minOccurs="0" name="ProcTemplValidFromDate"/>
          <element type="xsd:string" minOccurs="0" name="Icon"/>
          <element type="xsd:string" minOccurs="0" name="Category"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstSuspensionTime"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstSuspensionExpirationTime"/>
          <element type="xsd:long" minOccurs="0" name="Rc"/>
          <element type="xsd:string" minOccurs="0" name="MessageText"/>
          <element type="xsd:string" minOccurs="0" name="ProcTemplName"/>
          <element type="xsd:string" minOccurs="0" name="ProcInstName"/>
          <element type="xsd:long" minOccurs="0" name="ProgramRC"/>
          <element type="xsd:string" minOccurs="0" name="ExternalProcessContext"/>
        </sequence>
      </complexType>
    </xsl:if>
  </xsl:template>
  <!--
    ****************************************************************************************************************
    Template "program-activity-type":

    Determines the type of an FDL program activity.
    Selects from types "empty activity", "service invocation activity" (UPES activity),  "staff activity",
    "assumed staff activity"
    NOTE THAT THIS TEMPLATE IS SHARED IN:
      "XFDLBPEL_ANALYZE.XSL", "XFDLBPEL_WSDL.XSL", "XFDLBPEL_BPEL.XSL", "XFDLBPEL_SCDL.XSL"
    ****************************************************************************************************************
   -->
  <xsl:template name="program-activity-type">
    <xsl:param name="activity"/>
    <xsl:param name="server-may-be-missing" select="false()"/>
    <!-- ASSERT PRECONDITION -->
    <xsl:if test="not($activity)">
      <xsl:variable name="error-message" select="concat('PRECONDITION FAILED in template: name=&quot;program-activity-type&quot;', $eol, 'Parameter &quot;activity&quot; is not set')"/>
      <xsl:variable name="print-error-message" select="java:com.ibm.bpe.fdl2bpel.converter.CEnv.writeAssertion($error-message)"/>
    </xsl:if>
    <xsl:variable name="domain" select="//fdl:domain[1]"/>
    <xsl:variable name="system-group" select="//fdl:group[1]"/>
    <xsl:variable name="primary-system" select="//fdl:system[ @primarySystem ='yes' ][1]"/>
    <!-- Note that the following variables will contain appropriate values for "service invocation activities" (UPES) only -->
    <xsl:variable name="program-execution-unit" select="$activity/fdl:program/@programExecutionUnit"/>
    <xsl:variable name="server-name" select="substring-before($program-execution-unit, '.')"/>
    <xsl:variable name="related-system-name" select="substring-before(substring-after($program-execution-unit, concat($server-name, '.')), '.')"/>
    <xsl:variable name="server-and-system-name" select="concat($server-name, $related-system-name)"/>
    <xsl:choose>
      <!--
        CASE: "empty activity"
          A program activity is an "empty activity" if
          1) if the activity implementation is a program with the name FMCINTERNALNOOP
          2) the execution mode is chained
          3) the input and output data structures are the same
      -->
      <xsl:when test="$activity/fdl:program/@name='FMCINTERNALNOOP'
                and $activity/fdl:program/@synchronization='chained'
                and $activity/fdl:input/@type = $activity/fdl:output/@type">
        <xsl:value-of select="'empty activity'"/>
      </xsl:when>
      <!--
        CASE: "service invocation activity"
          A program activity is an "service invocation activity" if a user-defined program execution server
          (UPES) is specified (attribute "fdl:program/@programExecutionUnit").
      -->
      <xsl:when test="$activity/fdl:program/@programExecutionUnit
                  and (key('server-by-server-and-system-name', $server-and-system-name)[1]/fdl:userDefinedProgramExecutionServer or $server-may-be-missing = 'true')">
        <xsl:choose>
          <!-- CASE: User option "-un" (no UPES migration) is selected (cp. LI754 (Do not prepare process for reuse of UPES) -->
          <xsl:when test="'TRUE' = translate($createUPESInvocations, $lower-case-characters, $upper-case-characters)">
            <xsl:value-of select="'service invocation activity'"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="'empty activity'"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!--
        CASE: "staff activity"
          A program activity is a "staff activity" if the following conditions apply:
          1. Start type is "manual"
          2. Attribute "checkoutPossible" indicates that it can be checked out from the runtime database.
      -->
      <xsl:when test="$activity/fdl:join/@startType='manual'
                  and ( $activity/fdl:defaultSetting/@checkoutPossible='yes'
                       or (  not($activity/fdl:defaultSetting/@checkoutPossible='no')
                          and (   $activity/ancestor::fdl:process/fdl:activityDefaultSetting/@checkoutPossible='yes'
                              or (   'TRUE' = translate($LI1274_Support_inheritance_from_network_settings, $lower-case-characters, $upper-case-characters)
                                  and not($activity/ancestor::fdl:process/fdl:activityDefaultSetting/@checkoutPossible='no')
                                  and (  $primary-system/fdl:activityDefaultSetting/@checkoutPossible='yes'
                                      or (  not($primary-system/fdl:activityDefaultSetting/@checkoutPossible='no')
                                          and (   $system-group/fdl:activityDefaultSetting/@checkoutPossible='yes'
                                              or (  not($system-group/fdl:activityDefaultSetting/@checkoutPossible='no')
                                                  and $domain/fdl:activityDefaultSetting/@checkoutPossible='yes' ) ) ) ) ) ) ) )">
        <xsl:value-of select="'staff activity'"/>
      </xsl:when>
      <!--
        OTHERWISE: "unclassified activity"
          A program activity is an "unclassified activity" if none of the above rules apply.
          In this case assume a "staff activity".
      -->
      <xsl:otherwise>
        <xsl:value-of select="'assumed staff activity'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:transform>
