/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.command;

import java.util.List;

import com.PureEdge.error.UWIException;
import com.ibm.bpc.clientcore.ClientException;
import com.ibm.bpc.clientcore.Command;
import com.ibm.wbit.tel.client.form.data.FormDataConversionException;

/**
 * This class is a wrapper for an arbitrary command of type
 * <code>com.ibm.bpc.clientcore.Command</code>.
 */
abstract class CommandAdapter {

	private Command command;

	CommandAdapter(Command command) {
		this.command = command;
	}

	/**
	 * Execute the command.
	 * 
	 * @return return value of command
	 * @throws ClientException
	 * @throws UWIException
	 * @throws FormDataConversionException 
	 */
	String execute() throws ClientException, UWIException, FormDataConversionException {
		command.setContext(getContext());
		return command.execute(getInstances());
	}

	/**
	 * Get the input list for command execution.
	 * 
	 * @return list of command input parameters
	 * @throws UWIException
	 * @throws FormDataConversionException 
	 */
	abstract List getInstances() throws UWIException, FormDataConversionException;

	/**
	 * Get the context of the command.
	 * 
	 * @return command context
	 */
	abstract Object getContext();
}
