/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.command;

import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.ibm.bpc.clientcore.ClientException;
import com.ibm.bpc.clientcore.Command;
import com.ibm.task.api.ClientSetting;
import com.ibm.task.api.CustomClientSettings;
import com.ibm.task.api.HumanTaskManagerService;
import com.ibm.wbit.tel.client.form.data.FormDataConversionException;
import com.ibm.wbit.tel.client.form.decorator.FormDecoratorFacade;
import com.ibm.wbit.tel.client.form.persistence.PersistenceInfo;
import com.ibm.wbit.tel.client.form.persistence.PersistenceService;
import com.ibm.wbit.tel.client.jsf.infrastructure.CommandException;
import com.ibm.wbit.tel.client.jsf.infrastructure.ServiceFactory;
import com.ibm.wbit.tel.client.jsf.infrastructure.SharedConstants;

/**
 * Adds the form persistence logic to the command execution.
 */
abstract class FormCommand extends CommandAdapter {

	protected static final String XFDL_FILE_EXT = ".xfdl";

	private static final String LOTUS_FORMS_CLIENT = "Lotus Forms";

	private static final String PERSISTENCE_PARAM = "storeFormPersistently";

	protected FormNodeP form;

	FormCommand(Command command, FormNodeP form) {
		super(command);
		this.form = form;
	}

	/*
	 * Execute the command and save the form using the <code>PersistenceService</code>
	 * class.
	 * 
	 * @see com.ibm.wbit.tel.client.form.command.CommandAdapter#execute()
	 */
	String execute() throws ClientException, UWIException, FormDataConversionException {
		PersistenceInfo persistenceInfo = createPersistenceInfoBean();
		String returnPage = super.execute();
		saveForm(persistenceInfo);
		return returnPage;
	}

	/**
	 * Save the form, if persistence for the current task is enabled.
	 * 
	 * @param persistenceInfo
	 * @throws ClientException
	 * @throws UWIException
	 */
	void saveForm(PersistenceInfo persistenceInfo) throws ClientException, UWIException {
		try {
			// Check if there is a lotus forms client for the current task.
			HumanTaskManagerService htm = ServiceFactory.getInstance().getHumanTaskManagerService();
			CustomClientSettings allClients = htm.getUISettings(persistenceInfo.getTaskTemplateId());
			if (allClients != null) {
				ClientSetting formsClient = allClients.getClientSetting(LOTUS_FORMS_CLIENT);
				if (formsClient != null) {
					// Check if persistence is enabled.
					String storeFormPersistently = formsClient.getCustomSetting(PERSISTENCE_PARAM);
					boolean isPersistent = Boolean.parseBoolean(storeFormPersistently);
					if (isPersistent) {
						PersistenceService persService = new PersistenceService();

						// Remove buttons from form before it is persisted.
						FormDecoratorFacade formDecoratorFacade = new FormDecoratorFacade(form);
						formDecoratorFacade.remove();
						FormNodeP form = formDecoratorFacade.getForm();

						// Use the persistence service to save the form.
						persService.saveForm(persistenceInfo, form);
						persService.destroy();
					}
				}
			}
		} catch (Exception e) {
			throw new CommandException(SharedConstants.NLS_CATALOG, SharedConstants.FAILED_TO_PERSIST_FORM, null, e);
		}
	}

	/**
	 * This method will build the correct location for the form out of the subview id
	 * @param formLocation
	 * @return
	 */
	protected String getFormLocation(String formLocation){		
		// cut off "form"
		formLocation = formLocation.substring(7);
		// cut off _input
		if(formLocation.endsWith(SharedConstants.SUFFIX_JSF_PAGE_WRAPPING_INPUT_FORM)){
			int size = formLocation.length() - SharedConstants.SUFFIX_JSF_PAGE_WRAPPING_INPUT_FORM.length();
			formLocation = formLocation.substring(0, size);
		}
		// cut off _output
		if(formLocation.endsWith(SharedConstants.SUFFIX_JSF_PAGE_WRAPPING_OUTPUT_FORM)){
			int size = formLocation.length() - SharedConstants.SUFFIX_JSF_PAGE_WRAPPING_OUTPUT_FORM.length();
			formLocation = formLocation.substring(0, size);
		}
		return formLocation+ FormCommand.XFDL_FILE_EXT;
	}
	/**
	 * Specify the persistence information.
	 * 
	 * @return persistence information
	 */
	abstract PersistenceInfo createPersistenceInfoBean();
}
