/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.command;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.ibm.bpc.clientcore.ClientException;
import com.ibm.bpc.clientcore.Command;
import com.ibm.wbit.tel.client.form.data.FormDataConversionException;
import com.ibm.wbit.tel.client.form.data.FormDataObject;
import com.ibm.wbit.tel.client.form.persistence.PersistenceInfo;
import com.ibm.wbit.tel.client.jsf.bean.BCCreateInstance;
import com.ibm.wbit.tel.client.jsf.handler.BCCreateMessageHandler;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;

/**
 * Creates a task based on form data.
 */
class FormCreateCommand extends FormCommand {

	private BCCreateMessageHandler bcCreateMessageHandler;

	private PersistenceInfo persistenceInfo;
	
	private boolean isInputQualsOutput=false;

	FormCreateCommand(Command cmd, FormNodeP form, boolean isInputQualsOutput) throws UWIException {
		super(cmd, form);
		this.bcCreateMessageHandler = (BCCreateMessageHandler) FacesUtils.getManagedBeanInstance("bcCreateMessageHandler", BCCreateMessageHandler.class);
		this.persistenceInfo = new PersistenceInfo();
		this.isInputQualsOutput = isInputQualsOutput;
	}

	/*
	 * Fill execution input with form data.
	 */
	List getInstances() throws UWIException, FormDataConversionException {
		FormDataObject formDataObject = new FormDataObject(form);
		BCCreateInstance bcCreateInstance = bcCreateMessageHandler.getBcCreateInstance();
		
		String prefix = bcCreateInstance.getTaskID();
		// if input == output convert output xpath to input map.
		HashMap inputValues = formDataObject.getValues(prefix);
		if(isInputQualsOutput)
		{
			try {
				HashMap values = FacesUtils.getValueMapOutIn(inputValues, prefix);
				if(values != null)
				{
					inputValues = values;
				}
			} catch (ClientException e) {
				
			}
		}
		bcCreateInstance.setInputValuesAll(inputValues);
		List<BCCreateInstance> instances = new ArrayList<BCCreateInstance>();
		instances.add(bcCreateInstance);
		return instances;
	}

	/* (non-Javadoc)
	 * @see com.ibm.wbit.tel.client.form.command.FormCommand#createPersistenceInfoBean()
	 */
	PersistenceInfo createPersistenceInfoBean() {
		// Get form location from subview map.
		HashMap subviewMap = bcCreateMessageHandler.getSubviewMap();
		BCCreateInstance bcCreateInstance = bcCreateMessageHandler.getBcCreateInstance();
		String formLocation = (String) subviewMap.get(bcCreateInstance.getQNameInputMessage());
		// Remove the suffix "forms/" from the location.
		formLocation = getFormLocation(formLocation);
		persistenceInfo.setFormLocation(formLocation);
		return persistenceInfo;
	}

	/*
	 * The create command uses this context to store containment context and
	 * task template id.
	 */
	Object getContext() {
		return this.persistenceInfo;
	}
}
