/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.command;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.ibm.bpc.clientcore.Command;
import com.ibm.wbit.tel.client.form.data.FormDataConversionException;
import com.ibm.wbit.tel.client.form.data.FormDataObject;
import com.ibm.wbit.tel.client.form.persistence.PersistenceInfo;
import com.ibm.wbit.tel.client.jsf.bean.ToDoInstance;
import com.ibm.wbit.tel.client.jsf.handler.ToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;
import com.ibm.wbit.tel.client.jsf.infrastructure.SharedConstants;

/**
 * Handles a todo command based on form data.
 */
class FormToDoCommand extends FormCommand {

	private static final String SUBVIEW_MAP_SUFFIX = "output" + SharedConstants.CLIENT_TYPE_FORMS;

	private ToDoMessageHandler toDoMessageHandler;
	
	private boolean isMessageEmpty = false;

	FormToDoCommand(Command command, FormNodeP form, boolean isMessageEmpty) {
		super(command, form);
		this.toDoMessageHandler = (ToDoMessageHandler) FacesUtils.getManagedBeanInstance("toDoMessageHandler", ToDoMessageHandler.class);
		this.isMessageEmpty = isMessageEmpty;
	}

	/*
	 * Fill the execution input with form data.
	 */
	List getInstances() throws UWIException, FormDataConversionException {
		FormDataObject formDataObject = new FormDataObject(form);
		ToDoInstance todo = toDoMessageHandler.getToDoInstance();
	
		String prefix = FacesUtils.getHumanTaskID(todo);
		if(!this.isMessageEmpty){
			todo.setOutputValuesAll(formDataObject.getValues(prefix));
		}
		List<ToDoInstance> instances = new ArrayList<ToDoInstance>();
		instances.add(todo);
		return instances;
	}

	/* (non-Javadoc)
	 * @see com.ibm.wbit.tel.client.form.command.FormCommand#createPersistenceInfoBean()
	 */
	PersistenceInfo createPersistenceInfoBean() {
		PersistenceInfo persistenceInfo = new PersistenceInfo(false);

		// Set task template id.
		ToDoInstance todo = toDoMessageHandler.getToDoInstance();
		persistenceInfo.setTaskTemplateId(todo.getTaskTemplateID());

		// Get form location from subview map.
		HashMap subviewMap = toDoMessageHandler.getSubviewMap();
		String formLocation = (String) subviewMap.get(todo.getDefinitionNamespace() + todo.getDefinitionName() + SUBVIEW_MAP_SUFFIX);
		if (formLocation != null) {
			formLocation = getFormLocation(formLocation);
			persistenceInfo.setFormLocation(formLocation);
		}

		return persistenceInfo;
	}

	/* (non-Javadoc)
	 * @see com.ibm.wbit.tel.client.form.command.CommandAdapter#getContext()
	 */
	Object getContext() {
		return this.toDoMessageHandler;
	}
}
