/*
*
* Sample program for use with the Product
* Licensed Materials  - Property of IBM
* 5724-I66
* (c) Copyright IBM Corp.  2006, 2009
*   
*/
package com.ibm.wbit.tel.client.form.data;

import java.io.Serializable;
import java.util.List;

import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;
import com.ibm.wbit.tel.client.jsf.infrastructure.Messages;

/**
 * FormDataConversionException will be thrown if the Form input cannot be convered into the
 * expected java type. It will contain a detailed information why the conversion failed.
 */
public class FormDataConversionException extends Exception implements Serializable{
	
	private static final long serialVersionUID = -3922039197259914235L;
	
	private List<FormDataConversionException> errors;
	
	private String input;
	
	private String type;

	private String formString;
	
	public FormDataConversionException(String message){
		super(message);
	}
		
	public FormDataConversionException(){
		super(Messages.getString("FORM_ERROR_CONVERSION", FacesUtils.getLocale()));
	}

	public FormDataConversionException(String message, String input, String type){
		super(message);
		setInput(input);
		setType(type);
	}
	
	public FormDataConversionException(String message, Throwable e){
		super(message,e);
	}
	
	public FormDataConversionException(List<FormDataConversionException> errors){
		super(Messages.getString("FORM_ERROR_CONVERSION", FacesUtils.getLocale()));
		setErrors(errors);
	}

	public List<FormDataConversionException> getErrors() {
		return errors;
	}

	public void setErrors(List<FormDataConversionException> errors) {
		this.errors = errors;
	}

	public String getInput() {
		return input;
	}

	private void setInput(String input) {
		this.input = input;
	}

	public String getType() {
		return type;
	}

	private void setType(String type) {
		this.type = type;
	}
	
	public String getFormString() {
		return formString;
	}

	public void setFormString(String formString) {
		this.formString = formString;
	}


}
