/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2006, 2009
 *   
 */
package com.ibm.wbit.tel.client.form.data;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;
import com.ibm.wbit.tel.client.jsf.infrastructure.Messages;

/**
 * The FormDataConverter converts the input values out of the Lotus from into
 * java objects.
 */
public class FormDataConverter {

	/**
	 * The supported data types.
	 */
	public static final String xsd_boolean = "boolean";

	public static final String xsd_date = "date";

	public static final String xsd_dateTime = "dateTime";

	public static final String xsd_time = "time";

	public static final String xsd_gYear = "gYear";

	public static final String xsd_gYearMonth = "gYearMonth";

	public static final String xsd_gMonthDay = "gMonthDay";

	public static final String xsd_gDay = "gDay";
	
	public static final String xsd_duration = "duration";

	public static final String xsd_unsignedLong = "unsignedLong";

	public static final String xsd_positiveInteger = "positiveInteger";

	public static final String xsd_nonPositiveInteger = "nonPositiveInteger";

	public static final String xsd_nonNegativeInteger = "nonNegativeInteger";

	public static final String xsd_negativeInteger = "negativeInteger";

	public static final String xsd_integer = "integer";

	public static final String xsd_unsignedShort = "unsignedShort";

	public static final String xsd_int = "int";

	public static final String xsd_unsignedInt = "unsignedInt";

	public static final String xsd_unsignedByte = "unsignedByte";

	public static final String xsd_long = "long";

	public static final String xsd_short = "short";

	public static final String xsd_decimal = "decimal";

	public static final String xsd_byte = "byte";

	public static final String xsd_base64Binary = "base64Binary";

	public static final String xsd_hexBinary = "hexBinary";

	public static final String xsd_string = "string";

	public static final String xsd_float = "float";

	public static final String xsd_double = "double";

	public static final String xsd_anyURI = "anyURI";

	public static final String xsd_anySimpleType = "anySimpleType";
	
	public static final String xsd_any = "any";
	
	public static final String xsd_anyType = "anyType";

	/**
	 * This method converts the Form input form it's String representation into
	 * the right Java type. The mapping is based on the xsd to SDO 1.1 mapping.
	 * 
	 * @param input
	 *            The value from the Form element
	 * 
	 * @param xsdType
	 *            The xsd datatype.
	 * 
	 * @return The right Java object.
	 * 
	 * @throws FormDataConversionException
	 *             If the input value cannot be transformed to the specified
	 *             datatype this exception is thrown.
	 */
	public Object convertInput(String input, String type)
			throws FormDataConversionException {
		Object returnValue = null;
	
		if (input == null){			
			if(xsd_string.equals(type)) {
				return "";
			}
			if(xsd_boolean.equals(type)){
				return Boolean.FALSE;
			}			
			return null;
		} 
		
		if(xsd_any.equals(type)){
			return null;
		}
		
		if ("".equals(input)){			
			if(xsd_string.equals(type)) {
				return "";
			}
			if(xsd_boolean.equals(type)){
				return Boolean.FALSE;
			}			
			return null;
		} 
		
		try {
			if (xsd_boolean.equals(type)) {
				if("".equals(input)){
					returnValue = Boolean.FALSE;
				} else {
					returnValue = new Boolean(input);
				}

			} else if (xsd_date.equals(type)) {
				// date can be parsed if it has this format:
				//  - Sat Aug 11 00:00:00 CEST 2007    Regex: ([a-zA-Z]{3}\s){2}\d{2}\s\w{2}:\w{2}:\w{2}\s[a-zA-Z]{2,4}\s\w{4}
				//  - yyyy-MM-dd
				//  - yyyy.MM.dd
				
				if(input.matches("\\d{4}-\\d{2}-\\d{2}")){
					SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
					returnValue = format.parse(input);
				} else if(input.matches("\\d{4}\\.\\d{2}\\.\\d{2}")){
					SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd");
					returnValue = format.parse(input);
				} else if(input.matches("([a-zA-Z]{3}\\s){2}\\d{2}\\s\\w{2}:\\w{2}:\\w{2}\\s[a-zA-Z]{2,4}\\s\\w{4}")){
					SimpleDateFormat format = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzzz yyyy");
					returnValue = format.parse(input);
				}

			} else if (xsd_dateTime.equals(type)) {
				// dateTimemust have one of the following formats
				//  - 10-31-2005 06:45:21 PM 
				//  - 2000:12:21 12:12:12
				//  - 2000-12-21T12:23:24
				//  - 2000-12-21 12:23:24
				SimpleDateFormat format;
				if(input.matches("(0[1-9]|1[0-2])/((0[1-9])|([1-2][0-9])|(3[0-1]))/([0-9]{2,4}) (([0-1][0-9])|(2[0-3])):([0-5][0-9]):([0-5][0-9]) ((AM)|(PM))")){
					format = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss aa");
				}else if(input.matches("([0-9]{2,4})-(0[1-9]|1[0-2])-((0[1-9])|([1-2][0-9])|(3[0-1]))T(([0-1][0-9])|(2[0-3])):([0-5][0-9]):([0-5][0-9])")){
					format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
				} else 	if(input.matches("([0-9]{2,4})-(0[1-9]|1[0-2])-((0[1-9])|([1-2][0-9])|(3[0-1]))T(([0-1][0-9])|(2[0-3])):([0-5][0-9]):([0-5][0-9])Z")){
					format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
				} else if(input.matches("([0-9]{2,4})-(0[1-9]|1[0-2])-((0[1-9])|([1-2][0-9])|(3[0-1]))T(([0-1][0-9])|(2[0-3])):([0-5][0-9]):([0-5][0-9])(\\+|-)[0-5][0-9]:[0-5][0-9]")){
					input = input.substring(0, 22) + input.substring(23);
					format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
				}else if(input.matches("([0-9]{2,4})-(0[1-9]|1[0-2])-((0[1-9])|([1-2][0-9])|(3[0-1])) (([0-1][0-9])|(2[0-3])):([0-5][0-9]):([0-5][0-9])")){
					format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				} else{
					format = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss aa");
				}
			    TimeZone timeZone = TimeZone.getTimeZone("UTC");
			    format.setTimeZone(timeZone);
				returnValue = format.parse(input);
		

			} else if (xsd_time.equals(type)) {
				returnValue = input;

			} else if (xsd_gYear.equals(type)) {
				// gYear can be parsed if it has this format:
				// -yyyy
				if(input.matches("\\d{4}")){
					SimpleDateFormat format = new SimpleDateFormat("yyyy");
				    TimeZone timeZone = TimeZone.getTimeZone("UTC");
				    format.setTimeZone(timeZone);
					returnValue = format.parse(input);
				}

			} else if (xsd_gYearMonth.equals(type)) {
				// gYearMonth can be parsed if it has this format:
				//  - yyyy-mm						
				if(input.matches("\\d{4}-\\d{2}")){
					SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
				    TimeZone timeZone = TimeZone.getTimeZone("UTC");
				    format.setTimeZone(timeZone);
					returnValue = format.parse(input);
				}
			

			} else if (xsd_gMonthDay.equals(type)) {
				returnValue = input;

			} else if (xsd_gDay.equals(type)) {
				returnValue = input;
				
			} else if (xsd_duration.equals(type)) {
				returnValue = input;
				
				
			} else if (xsd_unsignedLong.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					returnValue = new BigInteger(input);
				}		

			} else if (xsd_positiveInteger.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					BigInteger bigInteger = new BigInteger(input);
					if(bigInteger.signum() > 0){
						returnValue = bigInteger;
					}
				}	

			} else if (xsd_nonNegativeInteger.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					BigInteger bigInteger = new BigInteger(input);
					if(bigInteger.signum() >= 0){
						returnValue = bigInteger;
					}
				}	

			} else if (xsd_negativeInteger.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					BigInteger bigInteger = new BigInteger(input);
					if(bigInteger.signum() <0){
						returnValue = bigInteger;
					}
				}	
		
				
			} else if (xsd_nonPositiveInteger.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					BigInteger bigInteger = new BigInteger(input);
					if(bigInteger.signum() <=0){
						returnValue = bigInteger;
					}
				}
			} else if (xsd_integer.equals(type)) {
				if("".equals(input)){
					returnValue = BigInteger.valueOf(0);
				} else {			
					returnValue = new BigInteger(input);
				}
			} else if (xsd_int.equals(type)) {
				if("".equals(input)){
					returnValue = Integer.valueOf(0);
				} else {
					returnValue = new Integer(input);
				}
			} else if (xsd_unsignedInt.equals(type)) {
				if("".equals(input)){
					returnValue = Long.valueOf(0);
				} else {			
					returnValue = new Long(input);
				}	

			} else if (xsd_unsignedByte.equals(type)) {
				if("".equals(input)){
					returnValue = 0;
				} else {			
					returnValue = new Short(input);
				}	
			
			} else if (xsd_unsignedShort.equals(type)) {
				if("".equals(input)){
					returnValue = Integer.valueOf(0);
				} else {			
					returnValue = new Integer(input);
				}	
			
				
			} else if (xsd_long.equals(type)) {
				if("".equals(input)){
					returnValue = Long.valueOf(0);
				} else {			
					returnValue = new Long(input);
				}	
			

			} else if (xsd_short.equals(type)) {
				if("".equals(input)){
					returnValue = 0;
				} else {			
					returnValue = new Short(input);
				}			

			} else if (xsd_decimal.equals(type)) {
				if("".equals(input)){
					returnValue = BigDecimal.valueOf(0d);
				} else {			
					input = replaceComma(input);
					returnValue = new BigDecimal(input);
				}			

			} else if (xsd_byte.equals(type)) {
				if("".equals(input)){
					byte i = 0;
					returnValue = Byte.valueOf(i);
				} else {			
					returnValue = new Byte(input);
				}						
				
			} else if (xsd_base64Binary.equals(type)) {
				returnValue = input.getBytes();
				
			} else if (xsd_hexBinary.equals(type)) {
				returnValue = input.getBytes();
				
			} else if (xsd_string.equals(type)) {
				returnValue = input;

			} else if (xsd_float.equals(type)) {
				if("".equals(input)){
					returnValue = Float.valueOf(0f);
				} else {			
					input = replaceComma(input);
					returnValue = new Float(input);
				}		
				
			} else if (xsd_double.equals(type)) {
				if("".equals(input)){
					returnValue = Double.valueOf(0d);
				} else {			
					input = replaceComma(input);
					returnValue = new Double(input);
				}		
				
			} else if (xsd_anyURI.equals(type)) { 
				returnValue = input;
				
			} else if (xsd_anySimpleType.equals(type)) {
				returnValue = input;
				
			} else if (xsd_any.equals(type)){
				returnValue = input;
			} else if (xsd_anyType.equals(type)){
				returnValue = input;
			}
			
		} catch (Exception e) {
			String message = Messages.bind("FORM_CONVERSION_WRONG_FORMAT", FacesUtils.getLocale(), new String[]{input, type});
			String hint = getHint(type);
			if(hint!=null){
				message = message.concat(hint);
			}
			throw new FormDataConversionException(message,input,type);
		}
		if (returnValue == null) {
			String message = Messages.bind("FORM_CONVERSION_WRONG_FORMAT", FacesUtils.getLocale(), new String[]{input, type});
			String hint = getHint(type);
			if(hint!=null){
				message = message.concat(hint);
			}
			throw new FormDataConversionException(message,input,type);
		}

		return returnValue;
	}

	private String replaceComma(String input) {
		if(input.contains(",")){
			input = input.replace(',', '.');
		}
		return input;
	}
	
	public String convertOutput(Object input, String type){
		// the default case.
		String returnValue = input.toString();
		
		if (xsd_date.equals(type) && input instanceof Date) {
			Date date = (Date) input;
			SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
			returnValue = dateFormatter.format(date);
			
		} else if (xsd_dateTime.equals(type) && input instanceof Date) {
			Date date = (Date) input;
			SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		    TimeZone timeZone = TimeZone.getTimeZone("UTC");
		    dateFormatter.setTimeZone(timeZone);
			returnValue = dateFormatter.format(date);

		} else if (xsd_gYear.equals(type) && input instanceof Date) {
			Date date = (Date) input;
			SimpleDateFormat dateFormatter  = new SimpleDateFormat("yyyy");
		    TimeZone timeZone = TimeZone.getTimeZone("UTC");
		    dateFormatter.setTimeZone(timeZone);
			returnValue = dateFormatter.format(date);
			
		} else if (xsd_gYearMonth.equals(type)  && input instanceof Date) {
			Date date = (Date) input;
			SimpleDateFormat dateFormatter  = new SimpleDateFormat("yyyy-MM");
		    TimeZone timeZone = TimeZone.getTimeZone("UTC");
		    dateFormatter.setTimeZone(timeZone);
			returnValue = dateFormatter.format(date);
			
		} else if (xsd_base64Binary.equals(type) && input instanceof byte[]){
			returnValue = new String((byte[])input);
			
		} else if (xsd_hexBinary.equals(type) && input instanceof byte[]){
			returnValue = new String((byte[])input);
		}


		return returnValue;
	}
	
	/**
	 * This method will return a hint for some types.
	 * @param type
	 * @return
	 */
	private String getHint(String type){
		String hint = null;
		
		if (xsd_date.equals(type)) {
			hint = Messages.getString("FORM_CONVERSION_HINT_DATE", FacesUtils.getLocale());			
		} else if (xsd_dateTime.equals(type)) {
			hint = Messages.getString("FORM_CONVERSION_HINT_DATETIME", FacesUtils.getLocale());		
		} else if (xsd_gYear.equals(type)){
			hint = Messages.getString("FORM_CONVERSION_HINT_GYEAR", FacesUtils.getLocale());			
		} else if (xsd_gYearMonth.equals(type)) {
			hint = Messages.getString("FORM_CONVERSION_HINT_GYEARMONTH", FacesUtils.getLocale());
		} 
		
		return hint;		
	}

}
