/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */
package com.ibm.wbit.tel.client.form.data;

/**
 * This class provides some simple methodes to manipulate xpath statements.
 */
class FormDataXpathUtil {
	
	/**
	 * This method returns the name of the attribute specified in the xpath statement,
	 * or null of there is xpath is not referencing an attribute.
	 * Example : "/a/@b" => "b"  and "/a/b" => null
	 * @param xpath
	 * 			The xpath pointing to an attribute node.
	 * @return
	 * 			The name of the attribute or null
	 */
	public String getAttributeName(String xpath){
		String attributeName=null;
		int pos = xpath.indexOf("@");
		if(pos >= 0 && pos < xpath.length()){
			attributeName = xpath.substring(pos +1);
		}
		return attributeName;		
	}
	
	/**
     * return the first part of an xpath. Works only for sdo xpaths
     * 
     * @param xpath
     * @return
     */
    public String getFirstPart(String xpath) {
        String firstPart = "";
        if (xpath != null && !"".equals(xpath)) {
            int pos = xpath.indexOf('/');
            if (pos == 0) {
                firstPart = getFirstPart(xpath.substring(1));
            } else if (pos != -1) {
                firstPart = xpath.substring(0, pos);
            } else {
                firstPart = xpath;
            }
        }
        return firstPart;
    }
    
	/**
     * This method will remove all Xpath statements that contain array predicates like
     * "[1]". 
     * Example: If the String "a/b[1]" is passed into this method the String "a/b" 
     * would be returned.
     * 
     * @param xpaths
     * 			A String object representing xpath statements.
     * @return
     * 		    A object representing no array xpaths.
     */
    public String removeArrayPredicatesFromXpath(String xpath){
		if(xpath.contains("[")){
			if(xpath.startsWith("/")){
				xpath = xpath.substring(1);
			}
			String[] parts = xpath.split("/");
			StringBuffer newXpath = new StringBuffer();
			for(String part: parts){
				if(part.contains("["))
				{
					part = part.substring(0, part.indexOf("["));
				}
				newXpath.append("/");
				newXpath.append(part);
			}
			return newXpath.toString();
		} 
		return xpath;
    }
    
    /**
     * removes the first element form an xpath statement. <br>
     * Example: /elemem1/elemem2 => /elemem2
     * 
     * @param xpath
     *            The xpath statement
     * @return The shorter xpath.
     */
    public String removeFirstPart(String xpath) {
        String lastPart = "";
        if (xpath != null && !"".equals(xpath)) {
            int pos = xpath.indexOf('/');
            if (pos == 0) {
                lastPart = removeFirstPart(xpath.substring(1));
            } else if (pos != -1) {
                lastPart = xpath.substring(pos);
            } else {
                // no / contained
                lastPart = "/".concat(xpath);
            }
        }
        return lastPart;
    }
    
	/**
	 * This method returns the position specified in the xpath. 
	 * Example: "a" => 1 and "a[2]" => 2 and "a[2]/b[3]" => 3
	 * 
	 * @param nodeName
	 * 			The nodename that might include a position information
	 * @return	
	 * 			The position.
	 */
	public int getNodePostion(String nodeName) {
		int position=1;
		if(nodeName.lastIndexOf("[") >=0){ // contains position information.
			String number = nodeName.substring(nodeName.lastIndexOf("[") +1);
			number = number.substring(0,number.lastIndexOf("]"));
			position = Integer.parseInt(number);
		}
		return position;
	}
	
	/**
	 * This method will return the last part of the xpath.
	 * Example: "/a/a" => "a" and "/a[2]" => "a[2]"
	 * @param xpath
	 * 			The xpath specifying the last part.
	 * @return
	 * 			The last part of the xpath. 
	 */			
	public String getNodeNameOfLastPart(String xpath) {
       String parentXpath = "";
        if (xpath != null && !"".equals(xpath)) {
            int pos = xpath.lastIndexOf('/');
            if (pos == 0) {
            	parentXpath = xpath.substring(1);
            } else if (pos == xpath.length()-1){
                	parentXpath = getNodeNameOfLastPart(xpath.substring(0, xpath.length()-1));
            } else if (pos != -1 && pos < xpath.length()) {
            	parentXpath = xpath.substring(pos +1);
            } 
        }     
        return parentXpath;	
	}

	/**
	 * This method retuns the first part of an xpath until an "[" appears.
	 * Example : "a[2]" => a
	 * 
	 * @param xpath
	 * 			The xpath
	 * @return
	 * 		The name of the node.
	 */
	public String getNodeName(String xpath){
        int pos = xpath.indexOf("[");
        if(pos >= 0){
        	xpath = xpath.substring(0, pos);
        }  
        return xpath;
	}
	
	/**
	 * This method will return the xpath of the parent node or null if it has no parent.
	 * Example: "/a/b[2]/c" => "/a/b[2]" and "/a" => null.
	 * @param xpath
	 * 			The xpath whichs parent to find out.
	 * @return
	 * 			The xpath to the parent node or null.
	 */
	public String removeLastPart(String xpath){
       String parentXpath = "";
        if (xpath != null && !"".equals(xpath)) {
            int pos = xpath.lastIndexOf('/');
            if (pos == 0) {
            	parentXpath = null;
            } else if (pos == xpath.length()-1){
                	parentXpath = removeLastPart(xpath.substring(0, xpath.length()-1));
            } else if (pos != -1) {
            	parentXpath = xpath.substring(0, pos);
            } 
        }
        return parentXpath;	
	}

	/**
	 * This method will return a xapth that is pointing to the first element in the entire array hierachy
	 * Example : a[2]/b[3] => a[1]/b[1]
	 * 
	 * @param parentXPath
	 * 			The xapth of a node
	 * @return
	 * 			The xpath with all array qualifiers set to 1.
	 */
	public String getTemplateXpath(String xpath) {
		if(xpath.contains("[")){
			if(xpath.startsWith("/")){
				xpath = xpath.substring(1);
			}
			String[] parts = xpath.split("/");
			StringBuffer newXpath = new StringBuffer();
			for(String part: parts){
				if(part.contains("["))
				{
					part = part.substring(0, part.indexOf("["));
					part +="[1]";
				}
				newXpath.append("/");
				newXpath.append(part);
				
			}
			return newXpath.toString();
		} 
		return xpath;
	}

}
