/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.decorator;

import java.util.Iterator;
import java.util.List;

import com.PureEdge.IFSSingleton;
import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.PureEdge.xfdl.XFDL;
import com.ibm.wbit.tel.client.form.FormUtils;
import com.ibm.wbit.tel.client.form.command.FormCommandFacade;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;
import com.ibm.wbit.tel.client.jsf.infrastructure.Messages;

/**
 * Adds buttons to the toolbar of each form page.
 */
class ButtonDecorator extends FormDecorator {

	static final int CREATE_BUTTON = 1;

	static final int SAVE_BUTTON = 2;

	static final int COMPLETE_BUTTON = 4;

	static final int RELEASE_BUTTON = 8;

	static final int SUBTASK_CREATE_BUTTON = 16;

	private static final String CREATE_BUTTON_ID = "IBM_BFM_CREATE_TASK";

	private static final String SAVE_BUTTON_ID = "IBM_BFM_SAVE_TASK";

	private static final String COMPLETE_BUTTON_ID = "IBM_BFM_COMPLETE_TASK";

	private static final String RELEASE_BUTTON_ID = "IBM_BFM_RELEASE_TASK";

	private static final String SUBTASK_CREATE_BUTTON_ID = "IBM_BFM_SUB_TODO_CREATE_TASK";

	private static final String LF_EXECUTE_JSP = "LF_Execute.jsp?";
	
	private static final String PARAMETER_METHOD = "&method=";

	private static final String PARAMETER_IME = "&ime=";
	
	private static final String PARAMETER_IEO = "ieo=";
	
	private int type;

	private String targetURL;
	
	ButtonDecorator(FormNodeP form, int type, boolean inputEqualsOutput) {
		this(form, type, inputEqualsOutput, false);
	}
	
	ButtonDecorator(FormNodeP form, int type, boolean inputEqualsOutput, boolean isMessageEmpty) {
		super(form);
		this.type = type;
		this.targetURL = FormUtils.createFacesServletURL() + LF_EXECUTE_JSP + PARAMETER_IEO + inputEqualsOutput + PARAMETER_IME + isMessageEmpty +PARAMETER_METHOD;
	}

	/*
	 * Add buttons of type <code>type</code>.
	 * 
	 * @see com.ibm.wbit.tel.client.form.decorator.FormDecorator#add()
	 */
	void add() throws UWIException {
		List pages = getPages();
		for (Iterator it = pages.iterator(); it.hasNext();) {
			FormNodeP page = (FormNodeP) it.next();
			FormNodeP toolbar = getToolbar(page);
			if (toolbar != null) {
				createButtons(toolbar);
			}
		}
	}

	/*
	 * Remove buttons of type <code>type</code>.
	 * 
	 * @see com.ibm.wbit.tel.client.form.decorator.FormDecorator#remove()
	 */
	void remove() throws UWIException {
		List pages = getPages();
		for (Iterator it = pages.iterator(); it.hasNext();) {
			FormNodeP page = (FormNodeP) it.next();
			if ((CREATE_BUTTON & type) > 0) {
				removeButton(page, CREATE_BUTTON_ID);
			}
			if ((SAVE_BUTTON & type) > 0) {
				removeButton(page, SAVE_BUTTON_ID);
			}
			if ((COMPLETE_BUTTON & type) > 0) {
				removeButton(page, COMPLETE_BUTTON_ID);
			}
			if ((RELEASE_BUTTON & type) > 0) {
				removeButton(page, RELEASE_BUTTON_ID);
			}
			if ((SUBTASK_CREATE_BUTTON & type) > 0) {
				removeButton(page, SUBTASK_CREATE_BUTTON_ID);
			}
		}
	}

	/**
	 * Remove a specific button from a form page.
	 * 
	 * @param page
	 *            page node
	 * @param id
	 *            button id
	 * @throws UWIException
	 */
	private void removeButton(FormNodeP page, String id) throws UWIException {
		FormNodeP button = getButton(page, id);
		if (button != null) {
			button.destroy();
		}
	}

	/**
	 * Get a specific button in a form page.
	 * 
	 * @param page
	 *            page node
	 * @param id
	 *            button id
	 * @return
	 * @throws UWIException
	 */
	private FormNodeP getButton(FormNodeP page, String id) throws UWIException {
		FormNodeP button = page.dereferenceEx(null, id, 0,
				FormNodeP.UFL_ITEM_REFERENCE, null);
		return button;
	}

	/**
	 * Get the toolbar of the form page.
	 * 
	 * @param page
	 *            page node
	 * @return toolbar node
	 * @throws UWIException
	 */
	private FormNodeP getToolbar(FormNodeP page) throws UWIException {
		FormNodeP toolbar = null;
		FormNodeP child = page.getChildren();
		while (child != null) {
			if ("toolbar".equals(child.getLocalName())) {
				toolbar = child;
			}
			child = child.getNext();
		}
		return toolbar;
	}

	/**
	 * Create buttons for all types.
	 * 
	 * @param toolbar
	 *            toolbar node
	 * @throws UWIException
	 */
	private void createButtons(FormNodeP toolbar) throws UWIException {
		int x = 10;
		int y = 10;
		String previousButtonId = null;

		if ((CREATE_BUTTON & type) > 0) {
			previousButtonId = createButton(toolbar, CREATE_BUTTON_ID, Messages
					.getString("BUTTON_CREATE", FacesUtils.getLocale()),
					this.targetURL + FormCommandFacade.CREATE_COMMAND, x, y,
					previousButtonId);
		}

		if ((COMPLETE_BUTTON & type) > 0) {
			previousButtonId = createButton(toolbar, COMPLETE_BUTTON_ID,
					Messages.getString("BUTTON_COMPLETE", FacesUtils
							.getLocale()), this.targetURL
							+ FormCommandFacade.COMPLETE_COMMAND, x, y,
					previousButtonId);
		}

		if ((SAVE_BUTTON & type) > 0) {
			previousButtonId = createButton(toolbar, SAVE_BUTTON_ID, Messages
					.getString("BUTTON_SAVE", FacesUtils.getLocale()),
					this.targetURL + FormCommandFacade.SAVE_COMMAND, x, y,
					previousButtonId);
		}

		if ((RELEASE_BUTTON & type) > 0) {
			previousButtonId = createButton(
					toolbar,
					RELEASE_BUTTON_ID,
					Messages
							.getString("BUTTON_RELEASE", FacesUtils.getLocale()),
					this.targetURL + FormCommandFacade.RELEASE_COMMAND, x, y,
					previousButtonId);
		}

		if ((SUBTASK_CREATE_BUTTON & type) > 0) {
			previousButtonId = createButton(
					toolbar,
					SUBTASK_CREATE_BUTTON_ID,
					Messages.getString("BUTTON_CREATE", FacesUtils.getLocale()),
					this.targetURL + FormCommandFacade.SUBTASK_CREATE_COMMAND,
					x, y, previousButtonId);
		}

	}

	/**
	 * Create a button for the given parameters.
	 * 
	 * @param toolbar
	 *            toolbar node
	 * @param id
	 *            button id
	 * @param value
	 *            button label
	 * @param url
	 *            target url
	 * @param x
	 *            horizontal location
	 * @param y
	 *            vertical location
	 * @param previousId
	 *            previous button id
	 * @return button id
	 * @throws UWIException
	 */
	private String createButton(FormNodeP toolbar, String id, String value,
			String url, int x, int y, String previousId) throws UWIException {
		FormNodeP pageNode = toolbar.getParent();
		FormNodeP submitButton = getButton(pageNode, id);
		if (submitButton == null) {
			submitButton = IFSSingleton.getXFDL().create(toolbar,
					XFDL.UFL_AFTER_SIBLING, "button", null, null, id);
			submitButton.setLiteralByRefEx(null, "itemlocation", 0, null, null,
					"");
			submitButton.setLiteralByRefEx(null, "itemlocation[within]", 0,
					null, null, toolbar.getIdentifier());
			if (previousId != null) {
				submitButton.setLiteralByRefEx(null, "itemlocation[after]", 0,
						null, null, previousId);
			}
			submitButton.setLiteralByRefEx(null, "value", 0, null, null, value);
			submitButton.setLiteralByRefEx(null, "type", 0, null, null, "done");
			submitButton.setLiteralByRefEx(null, "url", 0, null, null, url);
			submitButton.setLiteralByRefEx(null, "transmitformat", 0, null,
					null,
					"application/vnd.xfdl; content-encoding=\"base64-gzip\"");
		}

		return id;
	}
}
