/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.decorator;

import java.util.Map;

import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.ibm.bpc.clientcore.ClientException;
import com.ibm.wbit.tel.client.form.data.FormDataObject;
import com.ibm.wbit.tel.client.jsf.bean.ToDoInstance;
import com.ibm.wbit.tel.client.jsf.handler.BCDetailsHandler;
import com.ibm.wbit.tel.client.jsf.handler.SubToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.handler.ToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;

/**
 * Puts the data object to the form.
 */
public class DataObjectDecorator extends FormDecorator {

	static enum DATA_TYPE {
		TODO_INPUT, SUB_TODO_INPUT, TODO_OUTPUT, SUB_TODO_OUTPUT, DETAILS_INPUT, DETAILS_OUTPUT
	}

	private DATA_TYPE dataType;
	
	private String prefix;
	
	private boolean isInputEqualsOutput = false;

	DataObjectDecorator(FormNodeP form, DATA_TYPE dataType, String prefix, boolean isInputEqualsOutput) {
		super(form);
		this.dataType = dataType;
		this.prefix = prefix;
		this.isInputEqualsOutput = isInputEqualsOutput;
	}

	void add() throws UWIException {
		FormDataObject formDataObject = new FormDataObject(form);
		if (DATA_TYPE.TODO_OUTPUT.equals(dataType) || DATA_TYPE.SUB_TODO_OUTPUT.equals(dataType) || DATA_TYPE.DETAILS_OUTPUT.equals(dataType)) {
			formDataObject.setValues(getOutputValues(), prefix);
		} else {
			formDataObject.setValues(getInputValues(), prefix);
		}
	}

	void remove() throws UWIException {
	}

	/**
	 * Get input message values.
	 * 
	 * @return map of input values
	 */
	private Map getInputValues() {
		
		Map returnValues = null;
		if (DATA_TYPE.SUB_TODO_INPUT.equals(dataType)) {		
		
			SubToDoMessageHandler subToDoMessageHandler = (SubToDoMessageHandler) FacesUtils.getManagedBeanInstance("subToDoMessageHandler", SubToDoMessageHandler.class);
			ToDoInstance toDoInstance= subToDoMessageHandler.getSubToDoInstance();
			returnValues = getInputValuesFromToDo(toDoInstance);
			
		} else if (DATA_TYPE.TODO_INPUT.equals(dataType)) {

			ToDoMessageHandler toDoMessageHandler = (ToDoMessageHandler) FacesUtils.getManagedBeanInstance("toDoMessageHandler", ToDoMessageHandler.class);
			ToDoInstance toDoInstance = toDoMessageHandler.getToDoInstance();	
			returnValues = getInputValuesFromToDo(toDoInstance);
			
		} else if (DATA_TYPE.DETAILS_INPUT.equals(dataType)){	
			
			try{
				BCDetailsHandler bcDetailsHandler = (BCDetailsHandler) FacesUtils.getManagedBeanInstance("bcDetailsHandler", BCDetailsHandler.class);
				ToDoInstance toDoInstance= bcDetailsHandler.getInitiatingOTask();
				returnValues = getInputValuesFromToDo(toDoInstance);
			} catch (ClientException e) {}
			
		}		
		else {
			throw new IllegalArgumentException();
		}
		return returnValues;
	}

	/**
	 * Get output message values.
	 * 
	 * @return map of output values
	 */
	private Map getOutputValues() {
		
		Map outputValues = null;
		if (DATA_TYPE.TODO_OUTPUT.equals(dataType)) {
			
			ToDoMessageHandler toDoMessageHandler = (ToDoMessageHandler) FacesUtils.getManagedBeanInstance("toDoMessageHandler", ToDoMessageHandler.class);
			outputValues =  toDoMessageHandler.getToDoInstance().getOutputValues();
			
		} else if (DATA_TYPE.SUB_TODO_OUTPUT.equals(dataType)) {
			
			SubToDoMessageHandler subToDoMessageHandler = (SubToDoMessageHandler) FacesUtils.getManagedBeanInstance("subToDoMessageHandler", SubToDoMessageHandler.class);
			outputValues = subToDoMessageHandler.getSubToDoInstance().getOutputValues();
			
		} else if (DATA_TYPE.DETAILS_OUTPUT.equals(dataType)){	
			
				try{
					BCDetailsHandler bcDetailsHandler = (BCDetailsHandler) FacesUtils.getManagedBeanInstance("bcDetailsHandler", BCDetailsHandler.class);
					ToDoInstance toDoInstance= bcDetailsHandler.getInitiatingOTask();
					outputValues = toDoInstance.getOutputValues();
				} catch (ClientException e){}
				
		} else {
			throw new IllegalArgumentException();
		}
		return outputValues;
	}
	
	
	/**
	 * Helper method that will return the right map of input values for 
	 * an input form. If input == output the xpath will get converted to the
	 * output xath, since the form is allways the output form.
	 * 
	 * @param toDoInstance
	 * @return
	 */
	private Map getInputValuesFromToDo(ToDoInstance toDoInstance){
		if(isInputEqualsOutput){
			try{
				Map outputMap = FacesUtils.getValueMapInOut(toDoInstance);
				if(outputMap != null){
					return outputMap;
				}
			} catch (ClientException e){}
		}
		return toDoInstance.getInputValues();
	}
}
