/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.decorator;

import java.util.ArrayList;
import java.util.List;

import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;

/**
 * A decorator for a form, i.e. it adds/removes elements to it.
 */
abstract class FormDecorator {

	protected FormNodeP form;

	FormDecorator(FormNodeP form) {
		this.form = form;
	}

	/**
	 * Add objects to the form.
	 * 
	 * @throws UWIException
	 */
	abstract void add() throws UWIException;

	/**
	 * Remove objects from the form.
	 * 
	 * @throws UWIException
	 */
	abstract void remove() throws UWIException; 

	/**
	 * Collect all pages in the form.
	 * 
	 * @return list of page nodes
	 * @throws UWIException
	 */
	protected List<FormNodeP> getPages() throws UWIException {
		List<FormNodeP> pages = new ArrayList<FormNodeP>();

		FormNodeP child = this.form.getChildren().getNext();
		while (child != null) {
			if (child.getNodeType() == FormNodeP.UFL_PAGE) {
				pages.add(child);
			}
			child = child.getNext();
		}

		return pages;
	}
}
