/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.decorator;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;

import javax.faces.context.FacesContext;

import com.PureEdge.DTK;
import com.PureEdge.IFSSingleton;
import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.ibm.bpc.clientcore.ClientException;
import com.ibm.wbit.tel.client.form.persistence.PersistenceException;
import com.ibm.wbit.tel.client.form.persistence.PersistenceInfo;
import com.ibm.wbit.tel.client.form.persistence.PersistenceService;
import com.ibm.wbit.tel.client.jsf.bean.ToDoInstance;
import com.ibm.wbit.tel.client.jsf.handler.BCDetailsHandler;
import com.ibm.wbit.tel.client.jsf.handler.ToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;
import com.ibm.wbit.tel.client.jsf.infrastructure.Messages;

/**
 * A facade to add a decoration to a form.
 */
public class FormDecoratorFacade {

	/**
	 * Indicates which type of data should be filled into the form: input or
	 * output data. If <code>BOTH</code> is defined, input and output are the
	 * same and only on form is displayed on the work on pages.
	 */
	public static enum TYPE_KEY {
		INPUT, OUTPUT, BOTH, INPUT_NO_OUTPUT
	};

	private static final String BC_CREATE_VIEW_ID = "/BCCreateNew.jsp";

	private static final String TODO_CLAIM_VIEW_ID = "/ToDoClaim.jsp";

	private static final String TODO_WORK_ON_VIEW_ID = "/ToDoComplete.jsp";

	private static final String SUBTASK_USE_VIEW_ID = "/SubToDoDetails.jsp";

	private static final String SUBTASK_CREATE_VIEW_ID = "/SubToDoCreate.jsp";
	
	private static final String DETAILS_VIEW_ID = "/BCDetails.jsp";

	/**
	 * The name of the application calling initialize. This name is used to
	 * identify the application within the .ini file. It also sets the name that
	 * is returned by the XFDL applicationName function.
	 */
	private static final String PROGRAM_NAME = "*";

	/**
	 * The version number of the application calling initialize. If the .ini
	 * file has an entry for this version of the application, the application
	 * will bind to the version of the API listed in that entry.
	 */
	private static final String PROGRAM_VERSION = "1.0";

	/**
	 * The version number of the API the application should use by default. If
	 * the .ini file does not contain an entry for the specific application, the
	 * application will bind to the API specified by this parameter.
	 */
	private static final String API_VERSION = "3.0.0";

	private FormNodeP form;

	private TYPE_KEY type;

	private boolean isPersistedForm;
	
	private boolean isInputEqualsOutput;
	
	
	public FormDecoratorFacade(String xfdlRelativePath, URL xfdlURL, TYPE_KEY type, boolean isInputEqualsOutput) throws UWIException, PersistenceException, APIConfigException, IOException {
		this.type = type;
		this.isInputEqualsOutput = isInputEqualsOutput;
		initializeServerAPI();

		if(IFSSingleton.getXFDL() == null)
		{
			throw new APIConfigException(Messages.getString("SERVER_API_CONFIG", FacesUtils.getLocale()));
		}
		
		if (!isCurrentPage(BC_CREATE_VIEW_ID) && !isCurrentPage(SUBTASK_CREATE_VIEW_ID)) {
			PersistenceService persService = new PersistenceService();
			boolean isSubTask = isCurrentPage(SUBTASK_USE_VIEW_ID);
			PersistenceInfo persInfo = new PersistenceInfo(isSubTask);
			persInfo.setFormLocation(xfdlRelativePath);
			this.form = persService.getForm(persInfo);
			persService.destroy();
		}
		
		if (form == null) {
			this.form = IFSSingleton.getXFDL().readForm(xfdlURL.openStream(), 0);
			this.isPersistedForm = false;
		} else {
			this.isPersistedForm = true;
		}
	}

	public FormDecoratorFacade(String xfdlRelativePath, String xfdlAbsolutePath, TYPE_KEY type, boolean isInputEqualsOutput) throws UWIException, PersistenceException, APIConfigException {
		this.type = type;
		this.isInputEqualsOutput = isInputEqualsOutput;
		initializeServerAPI();

		if(IFSSingleton.getXFDL() == null)
		{
			throw new APIConfigException(Messages.getString("SERVER_API_CONFIG", FacesUtils.getLocale()));
		}
		
		if (!isCurrentPage(BC_CREATE_VIEW_ID) && !isCurrentPage(SUBTASK_CREATE_VIEW_ID)) {
			PersistenceService persService = new PersistenceService();
			boolean isSubTask = isCurrentPage(SUBTASK_USE_VIEW_ID);
			PersistenceInfo persInfo = new PersistenceInfo(isSubTask);
			persInfo.setFormLocation(xfdlRelativePath);
			this.form = persService.getForm(persInfo);
			persService.destroy();
		}
		
		if (form == null) {
			this.form = IFSSingleton.getXFDL().readForm(xfdlAbsolutePath, 0);
			this.isPersistedForm = false;
		} else {
			this.isPersistedForm = true;
		}
	}

	public FormDecoratorFacade(FormNodeP form) {
		this.form = form;
		this.type = TYPE_KEY.BOTH;
		this.isPersistedForm = true;
		this.isInputEqualsOutput = false;
	}

	/**
	 * Remove all generated buttons and toolbars from the form.
	 * 
	 * @throws UWIException
	 */
	public void remove() throws UWIException {
		FormDecorator buttonDecorator = new ButtonDecorator(form, ButtonDecorator.COMPLETE_BUTTON | ButtonDecorator.SAVE_BUTTON | ButtonDecorator.RELEASE_BUTTON | ButtonDecorator.CREATE_BUTTON, isInputEqualsOutput);
		buttonDecorator.remove();

		FormDecorator toolbarDecorator = new ToolbarDecorator(form);
		toolbarDecorator.remove();

		FormDecorator roDecorator = new ReadOnlyDecorator(form);
		roDecorator.remove();
	}

	public String getFormAsString() throws UWIException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		form.writeForm(baos, null, 0);
		baos.flush();
		return baos.toString("UTF8");
	}

	public FormNodeP getForm() {
		return this.form;
	}

	/**
	 * Add the decoration (toolbar, buttons, read-only nature, data object) to
	 * the form.
	 * 
	 * @throws UWIException
	 */
	public void add() throws UWIException {
		if (isCurrentPage(TODO_CLAIM_VIEW_ID)) {
			addDataObject(DataObjectDecorator.DATA_TYPE.TODO_INPUT);
			FormDecorator roDecorator = new ReadOnlyDecorator(form);
			roDecorator.add();
		} else if (isCurrentPage(BC_CREATE_VIEW_ID)) {
			FormDecorator toolbarDecorator = new ToolbarDecorator(form);
			toolbarDecorator.add();
			FormDecorator buttonDecorator = new ButtonDecorator(form, ButtonDecorator.CREATE_BUTTON,isInputEqualsOutput);
			buttonDecorator.add();
		} else if (isCurrentPage(SUBTASK_CREATE_VIEW_ID)) {
			FormDecorator toolbarDecorator = new ToolbarDecorator(form);
			toolbarDecorator.add();
			FormDecorator buttonDecorator = new ButtonDecorator(form, ButtonDecorator.SUBTASK_CREATE_BUTTON, isInputEqualsOutput);
			buttonDecorator.add();
		} else if (isCurrentPage(TODO_WORK_ON_VIEW_ID)) {
			if (TYPE_KEY.BOTH.equals(type) || TYPE_KEY.OUTPUT.equals(type)) {
				addDataObject(DataObjectDecorator.DATA_TYPE.TODO_OUTPUT);
				FormDecorator toolbarDecorator = new ToolbarDecorator(form);
				toolbarDecorator.add();
				FormDecorator buttonDecorator = new ButtonDecorator(form, ButtonDecorator.COMPLETE_BUTTON | ButtonDecorator.SAVE_BUTTON | ButtonDecorator.RELEASE_BUTTON,isInputEqualsOutput);
				buttonDecorator.add();
				FormDecorator roDecorator = new ReadOnlyDecorator(form);
				roDecorator.remove();
			} else {
				addDataObject(DataObjectDecorator.DATA_TYPE.TODO_INPUT);
				FormDecorator roDecorator = new ReadOnlyDecorator(form);
				roDecorator.add();
				
				// if only input, add the buttons.
				if(TYPE_KEY.INPUT_NO_OUTPUT.equals(type)){
					FormDecorator toolbarDecorator = new ToolbarDecorator(form);
					toolbarDecorator.add();
					FormDecorator buttonDecorator = new ButtonDecorator(form, ButtonDecorator.COMPLETE_BUTTON | ButtonDecorator.SAVE_BUTTON | ButtonDecorator.RELEASE_BUTTON,isInputEqualsOutput, true);
					buttonDecorator.add();					
				}
			}
		} else if (isCurrentPage(SUBTASK_USE_VIEW_ID)) {
			if (TYPE_KEY.BOTH.equals(type) || TYPE_KEY.OUTPUT.equals(type)) {
				addDataObject(DataObjectDecorator.DATA_TYPE.SUB_TODO_OUTPUT);
			} else {
				addDataObject(DataObjectDecorator.DATA_TYPE.SUB_TODO_INPUT);
			}
			FormDecorator roDecorator = new ReadOnlyDecorator(form);
			roDecorator.add();
		} else if (isCurrentPage(DETAILS_VIEW_ID)) {
			if (TYPE_KEY.BOTH.equals(type) || TYPE_KEY.OUTPUT.equals(type)) {
				addDataObjectForDetailsPage(DataObjectDecorator.DATA_TYPE.DETAILS_OUTPUT);
			} else {
				addDataObjectForDetailsPage(DataObjectDecorator.DATA_TYPE.DETAILS_INPUT);
			}
			FormDecorator roDecorator = new ReadOnlyDecorator(form);
			roDecorator.add();
		}
	}

	private void addDataObject(DataObjectDecorator.DATA_TYPE dataType) throws UWIException {
		// Do not overwrite the data in the persisted form!
		if (!isPersistedForm) {
			// Is this correct?
			ToDoMessageHandler toDoMessageHandler = (ToDoMessageHandler) FacesUtils.getManagedBeanInstance("toDoMessageHandler", ToDoMessageHandler.class);
			ToDoInstance todo = toDoMessageHandler.getToDoInstance();			
			String prefix = FacesUtils.getHumanTaskID(todo);		
			FormDecorator formDecorator = new DataObjectDecorator(form, dataType, prefix, isInputEqualsOutput);
			formDecorator.add();
		}
	}
	
	private void addDataObjectForDetailsPage(DataObjectDecorator.DATA_TYPE dataType)throws UWIException{
		
		if (!isPersistedForm) {
			// Is this correct? bcDetailsHandler
			try {
				BCDetailsHandler bcDetailsHandler = (BCDetailsHandler) FacesUtils.getManagedBeanInstance("bcDetailsHandler", BCDetailsHandler.class);
				ToDoInstance todo= bcDetailsHandler.getInitiatingOTask();
				String prefix = FacesUtils.getHumanTaskID(todo);
				FormDecorator formDecorator = new DataObjectDecorator(form, dataType, prefix, isInputEqualsOutput);
				formDecorator.add();
			} catch (ClientException e) {}
		}		
	}

	/**
	 * Destroy the form.
	 * 
	 * @throws UWIException
	 */
	public void destroy() throws UWIException {
		this.form.destroy();
	}

	private boolean isCurrentPage(String page) {
		String viewId = FacesContext.getCurrentInstance().getViewRoot().getViewId();
		if (viewId.equals(page)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * This method initializes the Server-API. You must call this method before
	 * calling any of the other methods in the Server-API.
	 * 
	 * @throws UWIException
	 */
	private void initializeServerAPI() throws UWIException {
		DTK.initialize(PROGRAM_NAME, PROGRAM_VERSION, API_VERSION);
	}
}
