/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.decorator;

import java.util.Iterator;
import java.util.List;

import com.PureEdge.IFSSingleton;
import com.PureEdge.error.UWIException;
import com.PureEdge.xfdl.FormNodeP;
import com.PureEdge.xfdl.XFDL;

/**
 * If there is no toolbar for a form page, this decorator creates one.
 */
class ToolbarDecorator extends FormDecorator {

	private static final String TOOLBAR_ID = "IBM_BFM_TOOLBAR";

	ToolbarDecorator(FormNodeP form) {
		super(form);
	}

	void add() throws UWIException {
		List pages = getPages();
		for (Iterator it = pages.iterator(); it.hasNext();) {
			FormNodeP page = (FormNodeP) it.next();
			FormNodeP toolbar = getToolbar(page, null);
			if (toolbar == null) {
				createToolbar(page);
			}
		}
	}

	/*
	 * Remove the toolbar only, if it was generated.
	 * 
	 * @see com.ibm.wbit.tel.client.form.decorator.FormDecorator#remove()
	 */
	void remove() throws UWIException {
		List pages = getPages();
		for (Iterator it = pages.iterator(); it.hasNext();) {
			FormNodeP page = (FormNodeP) it.next();
			FormNodeP toolbar = getToolbar(page, page.getIdentifier() + "_" + TOOLBAR_ID);
			if (toolbar != null) {
				toolbar.destroy();
			}
		}
	}

	/**
	 * Search for an existing toolbar.
	 * 
	 * @param page
	 *            page node
	 * @param id
	 *            toolbar id
	 * @return toolbar node
	 * @throws UWIException
	 */
	private FormNodeP getToolbar(FormNodeP page, String id) throws UWIException {
		FormNodeP toolbar = null;
		FormNodeP child = page.getChildren();
		while (child != null) {
			if ("toolbar".equals(child.getLocalName()) && (id == null || id.equals(child.getIdentifier()))) {
				toolbar = child;
			}
			child = child.getNext();
		}
		return toolbar;
	}

	/**
	 * Create a toolbar for a form page.
	 * 
	 * @param page
	 *            page node
	 * @throws UWIException
	 */
	private void createToolbar(FormNodeP page) throws UWIException {
		FormNodeP globalNode = page.dereferenceEx(null, "global", 0, FormNodeP.UFL_ITEM_REFERENCE, null);
		IFSSingleton.getXFDL().create(globalNode, XFDL.UFL_AFTER_SIBLING, "toolbar", null, null, page.getIdentifier() + "_" + TOOLBAR_ID);
	}
}
