/*
 *
 * Sample program for use with the Product
 * Licensed Materials  - Property of IBM
 * 5724-I66
 * (c) Copyright IBM Corp.  2008
 *   
 */

package com.ibm.wbit.tel.client.form.persistence;

import com.ibm.bpe.api.OID;
import com.ibm.bpe.api.PIID;
import com.ibm.task.api.TKTID;
import com.ibm.wbit.tel.client.jsf.bean.ToDoInstance;
import com.ibm.wbit.tel.client.jsf.handler.SubToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.handler.ToDoMessageHandler;
import com.ibm.wbit.tel.client.jsf.infrastructure.FacesUtils;

/**
 * Contains all necessary information for the persistence service: process
 * instance or task instance id, task template id and form location.
 */
public class PersistenceInfo {

	private OID containmentContextId;

	private TKTID taskTemplateId;

	private String formLocation;

	public PersistenceInfo(boolean isSubTask) {
		ToDoInstance todo;
		if (isSubTask) {
			SubToDoMessageHandler subToDoMessageHandler = (SubToDoMessageHandler) FacesUtils.getManagedBeanInstance("subToDoMessageHandler", SubToDoMessageHandler.class);
			todo = subToDoMessageHandler.getSubToDoInstance();
		} else {
			ToDoMessageHandler toDoMessageHandler = (ToDoMessageHandler) FacesUtils.getManagedBeanInstance("toDoMessageHandler", ToDoMessageHandler.class);
			todo = toDoMessageHandler.getToDoInstance();
		}
		if (todo != null) {
			OID id = todo.getID();
			OID parentOid = todo.getContainmentContextID();
			if (parentOid != null && parentOid instanceof PIID) {
				id = parentOid;
			}
			containmentContextId = id;
		}
	}

	public PersistenceInfo() {
		super();
	}

	public TKTID getTaskTemplateId() {
		return taskTemplateId;
	}

	public OID getContainmentContextId() {
		return containmentContextId;
	}

	public void setContainmentContextId(OID containmentContextId) {
		this.containmentContextId = containmentContextId;
	}

	public void setTaskTemplateId(TKTID tktid) {
		this.taskTemplateId = tktid;
	}

	public String getFormLocation() {
		return formLocation;
	}

	public void setFormLocation(String formLocation) {
		this.formLocation = formLocation;
	}
}
